import itertools
from typing import List, Tuple, Dict, Any, Optional

# ================== 1. 核心数据结构 ==================

class Node:
    """表达式树节点，用于构建AST"""
    def __init__(self, value: Any, left: Optional['Node'] = None, right: Optional['Node'] = None, op: Optional[str] = None):
        self.value = value  # 如果是叶子节点，则为数字；否则为操作符
        self.left = left
        self.right = right
        self.op = op
        
        # 用于存储优化信息
        self.strategy_name: Optional[str] = None
        self.strategy_score: int = 0
        self.result: Optional[float] = None

    def is_leaf(self) -> bool:
        """判断是否为叶子节点（即数字）"""
        return self.op is None

    def __repr__(self) -> str:
        """方便调试，打印表达式树"""
        if self.is_leaf():
            return str(self.value)
        return f"({self.left} {self.op} {self.right})"

# ================== 2. 策略库 ==================

class StrategyLibrary:
    """管理所有混合运算速算策略"""
    def __init__(self):
        self.strategies = [
            {"name": "凑整成百", "op": ['+', '-'], "level": "高", "score": 5, "func": self.is_round_to_100},
            {"name": "凑整成十", "op": ['+', '-'], "level": "高", "score": 4, "func": self.is_round_to_10},
            {"name": "小数凑整", "op": ['+'], "level": "中", "score": 2, "func": self.is_decimal_to_integer},
            {"name": "提取公因数", "op": ['*'], "level": "高", "score": 4, "func": self.can_extract_common_factor},
            {"name": "补数法", "op": ['+', '-'], "level": "中", "score": 3, "func": self.is_complementary},
            {"name": "倍数简化", "op": ['/'], "level": "中", "score": 3, "func": self.is_multiple_simplification},
        ]

    def get_best_strategy(self, node: Node) -> Optional[Dict]:
        """为给定的二元运算节点找到最佳策略"""
        if node.is_leaf() or not node.left or not node.right or not node.left.is_leaf() or not node.right.is_leaf():
            return None

        best_strategy = None
        for strategy in self.strategies:
            if node.op in strategy["op"] and strategy["func"](node.left.value, node.right.value):
                if not best_strategy or strategy["score"] > best_strategy["score"]:
                    best_strategy = strategy
        return best_strategy

    # --- 策略的具体实现 ---
    def is_round_to_100(self, a: float, b: float) -> bool:
        return (a + b) % 100 == 0

    def is_round_to_10(self, a: float, b: float) -> bool:
        return (a + b) % 10 == 0

    def is_decimal_to_integer(self, a: float, b: float) -> bool:
        """判断两个数（至少一个为小数）相加是否凑成整数"""
        # 检查至少一个数是小数
        a_is_decimal = isinstance(a, float) and not a.is_integer()
        b_is_decimal = isinstance(b, float) and not b.is_integer()
        if not (a_is_decimal or b_is_decimal):
            return False
        # 检查相加结果是否为整数
        result = a + b
        return isinstance(result, float) and result.is_integer()

    def can_extract_common_factor(self, a: float, b: float) -> bool:
        # 这是一个简化的例子，实际需要更复杂的树结构分析
        return False 

    def is_complementary(self, a: float, b: float) -> bool:
        return abs(a % 10 + b % 10 - 10) < 1e-9 or abs(a % 100 + b % 100 - 100) < 1e-9

    def is_multiple_simplification(self, a: float, b: float) -> bool:
        return b != 0 and a % b == 0

# ================== 3. 表达式优化器 ==================

class ExpressionOptimizer:
    """混合运算优化器，负责寻找最佳计算路径"""
    def __init__(self):
        self.strategy_lib = StrategyLibrary()

    def _parse_expression_with_structure(self, expr: str) -> dict:
        """解析表达式，保持结构信息以便进行高级优化"""
        import re
        
        # 移除空格
        expr = expr.replace(" ", "")
        
        # 使用正则表达式分割表达式，保留运算符
        tokens = re.split(r'([+\-*/])', expr)
        tokens = [token for token in tokens if token]  # 移除空字符串
        
        # 处理负数开头的情况
        if tokens and tokens[0] == '-':
            tokens = ['-' + tokens[1]] + tokens[2:]
        
        # 分析表达式结构，寻找优化模式
        optimization_found = self._find_optimization_patterns(tokens)
        
        if optimization_found:
            return optimization_found
        
        # 如果没有找到特殊优化模式，按常规方式处理
        return self._parse_conventional(tokens)
    
    def _find_optimization_patterns(self, tokens: List[str]) -> Optional[dict]:
        """寻找特殊的优化模式，如提取公因数等"""
        
        # 模式1: 提取公因数 (a*c + b*c = (a+b)*c)
        multiplication_terms = []
        addition_terms = []
        
        i = 0
        current_sign = 1
        
        while i < len(tokens):
            if tokens[i] in ['+', '-']:
                current_sign = 1 if tokens[i] == '+' else -1
                i += 1
                continue
            
            # 检查是否是乘法项
            if i + 2 < len(tokens) and tokens[i + 1] == '*':
                left = float(tokens[i])
                right = float(tokens[i + 2])
                multiplication_terms.append({
                    'left': left * current_sign,
                    'right': right,
                    'sign': current_sign
                })
                i += 3
            else:
                # 普通数字
                addition_terms.append(float(tokens[i]) * current_sign)
                i += 1
            
            current_sign = 1  # 重置符号
        
        # 检查是否有公因数
        if len(multiplication_terms) >= 2:
            common_factors = self._find_common_factors(multiplication_terms)
            if common_factors:
                return {
                    'type': 'common_factor',
                    'terms': multiplication_terms,
                    'addition_terms': addition_terms,
                    'common_factors': common_factors
                }
        
        return None
    
    def _find_common_factors(self, terms: List[dict]) -> Optional[dict]:
        """寻找乘法项中的公因数"""
        if len(terms) < 2:
            return None
            
        # 检查right部分是否有公因数
        right_values = [term['right'] for term in terms]
        if len(set(right_values)) == 1:  # 所有right值相同
            factor = right_values[0]
            coefficients = [term['left'] for term in terms]
            return {
                'factor': factor,
                'coefficients': coefficients,
                'position': 'right'
            }
        
        # 检查left部分是否有公因数
        left_values = [term['left'] for term in terms]
        if len(set(left_values)) == 1:  # 所有left值相同
            factor = left_values[0]
            coefficients = [term['right'] for term in terms]
            return {
                'factor': factor,
                'coefficients': coefficients,
                'position': 'left'
            }
        
        return None
    
    def _parse_conventional(self, tokens: List[str]) -> dict:
        """常规解析方式，先计算乘除法"""
        # 先计算所有乘除法
        i = 1  # 从第一个运算符开始
        while i < len(tokens):
            if i < len(tokens) and tokens[i] in ['*', '/']:
                left_val = float(tokens[i-1])
                operator = tokens[i]
                right_val = float(tokens[i+1])
                
                if operator == '*':
                    result = left_val * right_val
                else:  # operator == '/'
                    result = left_val / right_val
                
                # 替换这三个token为计算结果
                tokens = tokens[:i-1] + [str(result)] + tokens[i+2:]
            else:
                i += 2  # 跳过操作数和运算符
        
        # 处理加减法
        result_numbers = []
        current_sign = 1
        
        for i, token in enumerate(tokens):
            if token in ['+', '-']:
                current_sign = 1 if token == '+' else -1
            else:
                try:
                    num = float(token)
                    if i == 0:
                        result_numbers.append(num)
                    else:
                        result_numbers.append(current_sign * num)
                except ValueError:
                    continue
        
        return {
            'type': 'conventional',
            'numbers': result_numbers
        }

    def _build_tree_from_sequence(self, seq: List[float], op: str) -> Node:
        """从一个数字序列构建一个左深树"""
        if not seq:
            return Node(0)
        
        tree = Node(seq[0])
        for i in range(1, len(seq)):
            tree = Node(op, left=tree, right=Node(seq[i]), op=op)
        return tree

    def _evaluate_tree(self, node: Optional[Node]) -> Tuple[Optional[float], int]:
        """评估一棵表达式树的总奖励分数"""
        if not node:
            return 0.0, 0
        if node.is_leaf():
            return node.value, 0
        
        left_val, left_score = self._evaluate_tree(node.left)
        right_val, right_score = self._evaluate_tree(node.right)
        
        total_score = left_score + right_score
        
        # 计算当前节点结果
        if left_val is not None and right_val is not None:
            if node.op == '+':
                node.result = left_val + right_val
            elif node.op == '-':
                node.result = left_val - right_val
            # ...可以扩展其他操作符
        
        # 检查当前节点是否能应用策略
        strategy = self.strategy_lib.get_best_strategy(node)
        if strategy:
            node.strategy_name = strategy["name"]
            node.strategy_score = strategy["score"]
            total_score += strategy["score"]
            
        return node.result, total_score

    def _generate_associative_trees(self, numbers: Tuple[float, ...], op: str) -> List[Node]:
        """
        应用结合律，通过递归和缓存生成给定数字序列所有可能的二叉树结构。
        """
        if not numbers:
            return []
        if len(numbers) == 1:
            return [Node(numbers[0])]
        
        # 检查缓存
        if (numbers, op) in self.tree_cache:
            return self.tree_cache[(numbers, op)]

        all_trees = []
        # i 代表切分点，将 numbers 分为 left_part 和 right_part
        for i in range(1, len(numbers)):
            left_part = numbers[:i]
            right_part = numbers[i:]
            
            left_trees = self._generate_associative_trees(left_part, op)
            right_trees = self._generate_associative_trees(right_part, op)
            
            for l_tree in left_trees:
                for r_tree in right_trees:
                    all_trees.append(Node(op, left=l_tree, right=r_tree, op=op))
        
        # 存入缓存
        self.tree_cache[(numbers, op)] = all_trees
        return all_trees

    def _generate_candidates(self, numbers: List[float], op: str) -> List[Node]:
        """应用交换律和结合律生成所有可能的计算树"""
        self.tree_cache: Dict[Tuple, List[Node]] = {} # 为每次调用清空缓存
        candidates = []
        # 1. 应用交换律：生成所有排列
        # 为了减少计算量，对于大于6个数的排列，我们只取一部分样本
        num_perms = list(itertools.permutations(numbers))
        if len(numbers) > 6:
            num_perms = num_perms[:100] # 限制排列数量

        for perm in num_perms:
            # 2. 应用结合律：为每种排列生成所有可能的二叉树结构
            candidates.extend(self._generate_associative_trees(perm, op))
        
        # 去重，因为不同排列可能产生相同的树结构
        unique_candidates = []
        seen_trees = set()
        for tree in candidates:
            tree_repr = repr(tree)
            if tree_repr not in seen_trees:
                unique_candidates.append(tree)
                seen_trees.add(tree_repr)
        
        return unique_candidates


    def _generate_steps(self, node: Optional[Node]) -> List[str]:
        """从优化后的树生成人类可读的步骤"""
        if not node or node.is_leaf():
            return []
        
        # 深度优先遍历，后序生成步骤
        steps = self._generate_steps(node.left) + self._generate_steps(node.right)
        
        step_desc = f"{node.left} {node.op} {node.right} = {node.result}"
        if node.strategy_name:
            step_desc += f"  【{node.strategy_name}，+{node.strategy_score}分】"
        
        steps.append(step_desc)
        return steps

    def _collect_strategies(self, node: Optional[Node]) -> List[dict]:
        """收集树中所有使用的策略"""
        if not node:
            return []
        
        strategies = []
        
        # 收集左右子树的策略
        strategies.extend(self._collect_strategies(node.left))
        strategies.extend(self._collect_strategies(node.right))
        
        # 收集当前节点的策略
        if node.strategy_name:
            strategies.append({
                'name': node.strategy_name,
                'score': node.strategy_score,
                'description': f"{node.left} {node.op} {node.right} = {node.result}"
            })
        
        return strategies

    def optimize(self, expr: str):
        """主流程：解析、优化、评估并输出结果"""
        print(f"\n{'='*80}")
        print(f"                    小学数学速算 - 混合运算优化")
        print(f"{'='*80}")
        
        # 第1步：展示待计算问题算式
        print(f"📝 第一步：原始算式")
        print(f"   待计算表达式：{expr}")
        print()
        
        # 1. 解析表达式，识别优化模式
        parsed_result = self._parse_expression_with_structure(expr)
        
        if parsed_result['type'] == 'common_factor':
            # 处理公因数优化
            self._optimize_common_factor(expr, parsed_result)
        else:
            # 常规优化流程
            self._optimize_conventional(expr, parsed_result['numbers'])
    
    def _optimize_common_factor(self, expr: str, parsed_result: dict):
        """处理公因数优化"""
        common_factors = parsed_result['common_factors']
        terms = parsed_result['terms']
        addition_terms = parsed_result['addition_terms']
        
        factor = common_factors['factor']
        coefficients = common_factors['coefficients']
        
        # 第2步：优化后的算式，并解释优化点
        print(f"🎯 第二步：优化策略识别")
        print(f"   策略名称：提取公因数优化")
        print(f"   识别公因数：{factor}")
        print(f"   系数列表：{coefficients}")
        
        # 构建优化后的表达式
        coefficients_str = ' + '.join([str(c) if c >= 0 else f'({c})' for c in coefficients])
        optimized_expr = f"({coefficients_str}) × {factor}"
        if addition_terms:
            other_sum = sum(addition_terms)
            if other_sum >= 0:
                optimized_expr += f" + {other_sum}"
            else:
                optimized_expr += f" + ({other_sum})"
                
        print(f"   优化后算式：{optimized_expr}")
        print(f"   优化说明：将相同因数 {factor} 提取出来，简化乘法运算")
        print()
        
        # 第3步：展示对优化后算式的逐步计算结果
        print(f"🧮 第三步：逐步计算过程")
        
        # 计算系数和
        coeff_sum = sum(coefficients)
        print(f"   步骤1：计算系数和")
        coeffs_calculation = ' + '.join([str(c) if c >= 0 else f'({c})' for c in coefficients])
        print(f"          {coeffs_calculation} = {coeff_sum}")
        
        # 计算主要结果
        factored_result = coeff_sum * factor
        print(f"   步骤2：系数和乘以公因数")
        print(f"          {coeff_sum} × {factor} = {factored_result}")
        
        # 处理其他加法项
        final_result = factored_result
        if addition_terms:
            other_sum = sum(addition_terms)
            print(f"   步骤3：加上其他项")
            if other_sum >= 0:
                print(f"          {factored_result} + {other_sum} = {factored_result + other_sum}")
            else:
                print(f"          {factored_result} + ({other_sum}) = {factored_result + other_sum}")
            final_result = factored_result + other_sum
        
        print(f"   最终结果：{final_result}")
        print()
        
        # 第4步：评估与检查结果
        print(f"✅ 第四步：结果验证")
        try:
            standard_result = eval(expr)
        except:
            standard_result = final_result
            
        print(f"   原始算式计算：{expr} = {standard_result}")
        print(f"   优化算式计算：{optimized_expr} = {final_result}")
        print(f"   结果对比：{'✅ 正确' if abs(standard_result - final_result) < 1e-9 else '❌ 错误'}")
        
        # 计算节省的运算步骤
        original_ops = len([c for c in expr if c in '+-*/'])
        optimized_ops = len(coefficients) - 1 + 1 + (1 if addition_terms else 0)  # 系数相加 + 一次乘法 + 可能的加法
        saved_ops = original_ops - optimized_ops
        if saved_ops > 0:
            print(f"   效率提升：节省了 {saved_ops} 步运算")
        print()
        
        if abs(standard_result - final_result) < 1e-9:
            print("🎉 优化成功！算法正确应用了提取公因数策略。")
        else:
            print("❌ 优化失败！结果不匹配，请检查算法。")
    
    def _optimize_conventional(self, expr: str, numbers: List[float]):
        """常规优化流程"""
        op = '+'
        
        # 第2步开始：展示解析结果
        print(f"🔍 解析识别：数字列表 {numbers}")
        print()

        # 2. 生成所有候选计算路径（二叉树）
        candidate_trees = self._generate_candidates(numbers, op)

        # 3. 评估并选择最优方案
        best_tree = None
        max_score = -1
        all_strategies = []

        for tree in candidate_trees:
            _, score = self._evaluate_tree(tree)
            strategies = self._collect_strategies(tree)
            all_strategies.extend(strategies)
            if score > max_score:
                max_score = score
                best_tree = tree
        
        # 如果没有找到优化策略，使用原始顺序
        if not best_tree:
            best_tree = self._build_tree_from_sequence(numbers, op)
        
        # 重新评估最优树以填充结果
        self._evaluate_tree(best_tree)

        # 第2步：优化后的算式，并解释优化点
        print(f"🎯 第二步：优化策略识别")
        used_strategies = self._collect_strategies(best_tree)
        if used_strategies:
            print(f"   发现优化策略：")
            for strategy in used_strategies:
                print(f"     • {strategy['name']} (奖励分数: +{strategy['score']})")
                print(f"       应用位置: {strategy['description']}")
            print(f"   总优化得分：{max_score} 分")
        else:
            print(f"   未发现特殊优化策略，使用常规计算顺序")
            print(f"   总得分：{max_score} 分")
            
        print(f"   优化后算式：{best_tree}")
        print()

        # 第3步：展示对优化后算式的逐步计算结果  
        print(f"🧮 第三步：逐步计算过程")
        steps = self._generate_steps(best_tree)
        for i, step in enumerate(steps, 1):
            print(f"   步骤{i}：{step}")
        print(f"   最终结果：{best_tree.result}")
        print()

        # 第4步：评估与检查结果
        print(f"✅ 第四步：结果验证")
        try:
            standard_result = eval(expr)
        except:
            standard_result = sum(numbers)
            
        optimized_result = best_tree.result
        
        print(f"   原始算式计算：{expr} = {standard_result}")
        print(f"   优化算式计算：{best_tree} = {optimized_result}")
        print(f"   结果对比：{'✅ 正确' if optimized_result is not None and abs(standard_result - optimized_result) < 1e-9 else '❌ 错误'}")
        
        # 显示优化效果
        if max_score > 0:
            print(f"   优化效果：应用了 {len(used_strategies)} 个优化策略，总得分 {max_score} 分")
        else:
            print(f"   优化效果：未发现明显的优化机会")
        print()

        if optimized_result is None or abs(standard_result - optimized_result) > 1e-9:
            print("❌ 优化失败！结果不匹配或计算错误。")
        else:
            print("🎉 优化成功！算法正确应用了速算策略。")





"""
交换结合法加法实现
利用加法的交换律和结合律重新排列加数，使计算更简单
适用于能够重新组合得到更简单计算的情况
"""

from typing import List
from ..base_types import MathCalculator, Formula, CalculationStep


class CommutativeAssociativeAddition(MathCalculator):
    """交换结合法加法算法"""
    
    def __init__(self):
        super().__init__("交换结合法", "利用交换律和结合律重新排列，优化计算", priority=1)
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：适用于能通过重新排列简化的加法"""
        if formula.type != "addition":
            return False
        
        numbers = formula.get_numbers()
        if len(numbers) != 2:
            return False
        
        try:
            addend1, addend2 = [elem.get_numeric_value() for elem in numbers]
            if not (isinstance(addend1, (int, float)) and isinstance(addend2, (int, float))):
                return False
            
            # 转换为整数
            addend1, addend2 = int(addend1), int(addend2)
            
            # 只处理正数
            if addend1 <= 0 or addend2 <= 0:
                return False
            
            # 这是一个兜底算法，优先级最低
            # 主要用于演示交换律和结合律的概念
            # 只在其他更具体的算法都不适用时才使用
            
            # 检查是否有重新排列的价值
            # 例如：个位数字的重新组合能产生进位或简化
            
            # 简单示例：如果两个数都是两位数，检查是否可以通过交换获得更好的计算顺序
            if 10 <= addend1 <= 99 and 10 <= addend2 <= 99:
                # 获取各位数字
                a_tens, a_ones = addend1 // 10, addend1 % 10
                b_tens, b_ones = addend2 // 10, addend2 % 10
                
                # 如果重新组合后能得到更简单的计算，则适用
                # 例如：23 + 45 可以看作 (20+40) + (3+5) = 60 + 8 = 68
                # 但这种情况通常会被其他算法（如由高到低位法）覆盖
                
                # 这里我们只在非常特殊的情况下应用，作为演示用途
                return False  # 暂时禁用，避免与其他算法冲突
            
            return False
        except:
            return False
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建交换结合法步骤"""
        numbers = formula.get_numbers()
        addend1 = int(numbers[0].get_numeric_value())
        addend2 = int(numbers[1].get_numeric_value())
        
        steps = []
        
        steps.append(CalculationStep(
            description=f"{addend1} + {addend2} 使用交换结合法",
            operation="识别交换结合法",
            result="利用交换律和结合律重新排列"
        ))
        
        # 演示交换律
        steps.append(CalculationStep(
            description=f"交换律：{addend1} + {addend2} = {addend2} + {addend1}",
            operation="应用交换律",
            result=f"可以交换加数顺序"
        ))
        
        # 如果是两位数，演示分解和重新组合
        if 10 <= addend1 <= 99 and 10 <= addend2 <= 99:
            a_tens, a_ones = addend1 // 10, addend1 % 10
            b_tens, b_ones = addend2 // 10, addend2 % 10
            
            steps.append(CalculationStep(
                description=f"分解：{addend1} = {a_tens * 10} + {a_ones}，{addend2} = {b_tens * 10} + {b_ones}",
                operation="数字分解",
                result=f"按位分解后重新组合"
            ))
            
            steps.append(CalculationStep(
                description=f"结合律重组：({a_tens * 10} + {b_tens * 10}) + ({a_ones} + {b_ones})",
                operation="应用结合律",
                result=f"同类项结合"
            ))
            
            tens_sum = (a_tens + b_tens) * 10
            ones_sum = a_ones + b_ones
            
            steps.append(CalculationStep(
                description=f"分别计算：{a_tens * 10} + {b_tens * 10} = {tens_sum}，{a_ones} + {b_ones} = {ones_sum}",
                operation="分组计算",
                result=f"十位和：{tens_sum}，个位和：{ones_sum}"
            ))
            
            final_result = tens_sum + ones_sum
            steps.append(CalculationStep(
                description=f"合并结果：{tens_sum} + {ones_sum} = {final_result}",
                operation="合并结果",
                result=final_result,
                formula="交换结合法：(a+c)+(b+d) = (a+b)+(c+d)"
            ))
        else:
            # 简单情况直接计算
            final_result = addend1 + addend2
            steps.append(CalculationStep(
                description=f"直接计算：{addend1} + {addend2} = {final_result}",
                operation="直接相加",
                result=final_result,
                formula="交换结合法：a + b = b + a"
            ))
        
        return steps


"""
补零法除法实现（除以10的幂）
除以10, 100, 1000等，只需将被除数的小数点向左移动相应位数
"""

from typing import List
from ..base_types import MathCalculator, Formula, CalculationStep


class PowerOfTenDivision(MathCalculator):
    """补零法除法算法（除以10的幂）"""
    
    def __init__(self):
        super().__init__("补零法除法", "除以10的幂，小数点左移", priority=8)
    
    def _is_power_of_ten(self, num: int) -> bool:
        """检查是否为10的幂"""
        if num <= 0:
            return False
        
        str_num = str(num)
        return str_num[0] == '1' and all(d == '0' for d in str_num[1:])
    
    def _get_power_of_ten(self, num: int) -> int:
        """获取10的幂次"""
        if not self._is_power_of_ten(num):
            return 0
        return len(str(num)) - 1
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：除数是10的幂（10, 100, 1000...）"""
        if formula.type != "division":
            return False
        
        numbers = formula.get_numbers()
        if len(numbers) != 2:
            return False
        
        try:
            dividend, divisor = [elem.get_numeric_value() for elem in numbers]
            if not isinstance(divisor, (int, float)):
                return False
            
            # 除数必须是10的幂
            if isinstance(divisor, float):
                # 处理10.0这样的浮点数
                if divisor.is_integer():
                    divisor = int(divisor)
                else:
                    return False
            
            return self._is_power_of_ten(divisor)
            
        except:
            return False
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建补零法除法步骤"""
        numbers = formula.get_numbers()
        dividend = numbers[0].get_numeric_value()
        divisor = numbers[1].get_numeric_value()
        
        # 确保除数是整数
        if isinstance(divisor, float) and divisor.is_integer():
            divisor = int(divisor)
        
        power = self._get_power_of_ten(divisor)
        
        steps = []
        
        steps.append(CalculationStep(
            description=f"{dividend} ÷ {divisor} 使用补零法除法",
            operation="识别10的幂",
            result=f"{divisor} = 10^{power}，小数点左移{power}位"
        ))
        
        steps.append(CalculationStep(
            description=f"除以10^{power}等于小数点向左移动{power}位",
            operation="规律说明",
            result="补零法：除以10的幂 = 小数点左移"
        ))
        
        # 处理被除数
        if isinstance(dividend, int):
            dividend_str = str(dividend)
            if len(dividend_str) > power:
                # 整数位足够，直接移动小数点
                integer_part = dividend_str[:-power] if power > 0 else dividend_str
                decimal_part = dividend_str[-power:] if power > 0 else ''
                
                if power == 0:
                    result_str = dividend_str
                elif integer_part == '':
                    result_str = '0.' + '0' * (power - len(dividend_str)) + dividend_str
                else:
                    result_str = integer_part + ('.' + decimal_part if decimal_part else '')
                
                steps.append(CalculationStep(
                    description=f"移动小数点：{dividend_str} → {result_str}",
                    operation="小数点移动",
                    result=result_str
                ))
            else:
                # 需要在前面补0
                zeros_needed = power - len(dividend_str)
                result_str = '0.' + '0' * zeros_needed + dividend_str
                
                steps.append(CalculationStep(
                    description=f"位数不足，前面补零：{dividend_str} → {result_str}",
                    operation="补零移位",
                    result=result_str
                ))
        else:
            # 被除数是小数
            result = dividend / divisor
            steps.append(CalculationStep(
                description=f"小数除法：{dividend} ÷ {divisor} = {result}",
                operation="小数计算",
                result=result
            ))
        
        # 计算最终结果
        final_result = dividend / divisor
        
        # 如果结果是整数，显示为整数
        if isinstance(final_result, float) and final_result.is_integer():
            final_result = int(final_result)
        
        steps.append(CalculationStep(
            description=f"最终结果：{dividend} ÷ {divisor} = {final_result}",
            operation="确定结果",
            result=final_result,
            formula="a ÷ 10^n = a × 10^(-n)"
        ))
        
        return steps


"""
乘以125实现
利用125 = 1000/8的特点，转换为除以8再乘1000
"""

from typing import List
from ..base_types import MathCalculator, Formula, CalculationStep


class MultiplyByOneHundredTwentyFive(MathCalculator):
    """乘以125算法"""
    
    def __init__(self):
        super().__init__("乘125速算", "任意整数乘125，除以8再添三个0", priority=3)
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：任意数乘以125"""
        if formula.type != "multiplication":
            return False
        
        numbers = formula.get_numbers()
        if len(numbers) != 2:
            return False
        
        try:
            a, b = [elem.get_numeric_value() for elem in numbers]
            if not (isinstance(a, (int, float)) and isinstance(b, (int, float))):
                return False
            
            # 检查是否有一个数是125
            return a == 125 or b == 125
        except:
            return False
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建乘125速算步骤"""
        numbers = formula.get_numbers()
        a, b = [elem.get_numeric_value() for elem in numbers]
        
        # 确定哪个是125，哪个是被乘数
        if a == 125:
            multiplied_num = b
        else:
            multiplied_num = a
        
        steps = []
        
        steps.append(CalculationStep(
            description=f"{multiplied_num} × 125 使用乘125速算法",
            operation="识别模式",
            result="125 = 1000 ÷ 8，转换为除8乘1000"
        ))
        
        # 检查是否能被8整除
        if multiplied_num % 8 == 0:
            quotient = int(multiplied_num // 8)
            steps.append(CalculationStep(
                description=f"{multiplied_num} ÷ 8 = {quotient}",
                operation="除以8",
                result=f"{quotient}"
            ))
            
            final_result = quotient * 1000
            steps.append(CalculationStep(
                description=f"{quotient} × 1000 = {final_result}",
                operation="乘以1000",
                result=final_result,
                formula="a × 125 = (a ÷ 8) × 1000"
            ))
        else:
            # 不能被8整除，转换为小数计算
            quotient = multiplied_num / 8
            steps.append(CalculationStep(
                description=f"{multiplied_num} ÷ 8 = {quotient}",
                operation="除以8",
                result=f"{quotient}"
            ))
            
            final_result = quotient * 1000
            if final_result.is_integer():
                final_result = int(final_result)
            
            steps.append(CalculationStep(
                description=f"{quotient} × 1000 = {final_result}",
                operation="乘以1000",
                result=final_result,
                formula="a × 125 = (a ÷ 8) × 1000"
            ))
        
        return steps


"""
分拆法加法实现
将复杂的加法拆分成多个简单的加法步骤
适用于较大数字或复杂结构的加法，如137+268可拆分为100+200+30+60+7+8
"""

from typing import List
from ..base_types import MathCalculator, Formula, CalculationStep


class SplitAddition(MathCalculator):
    """分拆法加法算法"""
    
    def __init__(self):
        super().__init__("分拆法", "将复杂加法拆分成多个简单步骤", priority=3)
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：适用于三位数及以上或特殊结构的加法"""
        if formula.type != "addition":
            return False
        
        numbers = formula.get_numbers()
        if len(numbers) != 2:
            return False
        
        try:
            addend1, addend2 = [elem.get_numeric_value() for elem in numbers]
            if not (isinstance(addend1, (int, float)) and isinstance(addend2, (int, float))):
                return False
            
            # 转换为整数
            addend1, addend2 = int(addend1), int(addend2)
            
            # 适用于三位数及以上，或者两位数中有特殊拆分价值的
            if addend1 >= 100 or addend2 >= 100:
                return True
            
            # 两位数中，如果拆分后能产生更简单的计算，也适用
            # 例如：25+37可以拆分为20+30+5+7
            if addend1 >= 20 and addend2 >= 20:
                ones1, ones2 = addend1 % 10, addend2 % 10
                tens1, tens2 = addend1 // 10, addend2 // 10
                # 如果拆分后能产生整十数相加，则适用
                return (ones1 + ones2) != 0 and (ones1 + ones2) != 10
            
            return False
        except:
            return False
    
    def _split_number(self, number: int) -> List[tuple]:
        """将数字按位拆分，返回(位名称, 数值)的列表"""
        if number == 0:
            return [("个位", 0)]
        
        parts = []
        position_names = ["个位", "十位", "百位", "千位", "万位"]
        position = 0
        temp = number
        
        while temp > 0 and position < len(position_names):
            digit = temp % 10
            if digit > 0:
                place_value = digit * (10 ** position)
                parts.append((position_names[position], place_value))
            temp //= 10
            position += 1
        
        # 返回从高位到低位的顺序
        return list(reversed(parts))
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建分拆法步骤"""
        numbers = formula.get_numbers()
        addend1 = int(numbers[0].get_numeric_value())
        addend2 = int(numbers[1].get_numeric_value())
        
        steps = []
        
        steps.append(CalculationStep(
            description=f"{addend1} + {addend2} 使用分拆法",
            operation="识别分拆法",
            result="将数字按位分拆，逐步相加"
        ))
        
        # 分拆两个数字
        parts1 = self._split_number(addend1)
        parts2 = self._split_number(addend2)
        
        parts1_str = " + ".join([str(val) for _, val in parts1])
        parts2_str = " + ".join([str(val) for _, val in parts2])
        
        steps.append(CalculationStep(
            description=f"分拆数字",
            operation="数字分拆",
            result=f"{addend1} = {parts1_str}；{addend2} = {parts2_str}"
        ))
        
        # 重新组织算式
        all_parts = []
        all_parts.extend([val for _, val in parts1])
        all_parts.extend([val for _, val in parts2])
        
        reorganized = " + ".join([str(val) for val in all_parts])
        steps.append(CalculationStep(
            description=f"重新组织算式",
            operation="算式重组",
            result=f"原式 = {reorganized}"
        ))
        
        # 按位归类相加
        position_sums = {}
        for pos_name, val in parts1:
            if pos_name not in position_sums:
                position_sums[pos_name] = 0
            position_sums[pos_name] += val
        
        for pos_name, val in parts2:
            if pos_name not in position_sums:
                position_sums[pos_name] = 0
            position_sums[pos_name] += val
        
        # 按位次顺序计算
        position_order = ["万位", "千位", "百位", "十位", "个位"]
        partial_results = []
        
        for pos_name in position_order:
            if pos_name in position_sums:
                sum_val = position_sums[pos_name]
                partial_results.append(sum_val)
                
                # 找出该位的组成部分
                parts_in_position = []
                for pos, val in parts1:
                    if pos == pos_name:
                        parts_in_position.append(val)
                for pos, val in parts2:
                    if pos == pos_name:
                        parts_in_position.append(val)
                
                if len(parts_in_position) > 1:
                    parts_str = " + ".join([str(p) for p in parts_in_position])
                    steps.append(CalculationStep(
                        description=f"{pos_name}相加：{parts_str} = {sum_val}",
                        operation=f"{pos_name}计算",
                        result=sum_val
                    ))
                else:
                    steps.append(CalculationStep(
                        description=f"{pos_name}：{sum_val}（仅一个数有此位）",
                        operation=f"{pos_name}计算",
                        result=sum_val
                    ))
        
        # 合并所有位的结果
        final_result = sum(partial_results)
        if len(partial_results) > 1:
            partial_str = " + ".join([str(x) for x in partial_results])
            steps.append(CalculationStep(
                description=f"合并各位结果：{partial_str} = {final_result}",
                operation="合并结果",
                result=final_result,
                formula="分拆法：按位分拆后归类相加"
            ))
        else:
            steps.append(CalculationStep(
                description=f"最终结果：{final_result}",
                operation="确认结果",
                result=final_result,
                formula="分拆法：按位分拆后归类相加"
            ))
        
        return steps


"""
约分法除法实现
被除数和除数有公因数时，先同时除以它们的最大公因数
"""

from typing import List
from ..base_types import MathCalculator, Formula, CalculationStep
import math


class ReductionDivision(MathCalculator):
    """约分法除法算法"""
    
    def __init__(self):
        super().__init__("约分法", "找公因数，先约分", priority=3)
    
    def _find_common_factors(self, a: int, b: int) -> List[int]:
        """找到两个数的所有公因数"""
        gcd = math.gcd(abs(a), abs(b))
        factors = []
        
        for i in range(1, int(gcd**0.5) + 1):
            if gcd % i == 0 and i != b and i != 1:
                factors.append(i)
                if i != gcd // i:
                    factors.append(gcd // i)
        
        return sorted(set(factors), reverse=True)
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：被除数和除数都是整数，且有公因数大于1"""
        if formula.type != "division":
            return False
        
        numbers = formula.get_numbers()
        if len(numbers) != 2:
            return False
        
        try:
            dividend, divisor = [elem.get_numeric_value() for elem in numbers]
            
            # 必须都是整数
            if not (isinstance(dividend, int) and isinstance(divisor, int)):
                return False
            
            if divisor == 0:
                return False
            
            # 必须有大于1的公因数,且公因数不能是除数本身
            gcd = math.gcd(abs(dividend), abs(divisor))
            # 公因数判断难度较高，限制gcd为5或2的倍数
            if gcd % 5 != 0 and gcd % 2 != 0:
                return False
            gcc = self._find_common_factors(dividend, divisor)
            return gcd > 1 and len(gcc) > 0
        except:
            return False
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建约分法除法步骤"""
        numbers = formula.get_numbers()
        dividend = int(numbers[0].get_numeric_value())
        divisor = int(numbers[1].get_numeric_value())
        
        steps = []
        
        steps.append(CalculationStep(
            description=f"{dividend} ÷ {divisor} 使用约分法",
            operation="识别整数除法",
            result="找公因数，先约分"
        ))
        
        # 找到所有公因数
        common_factors = self._find_common_factors(dividend, divisor)

        gcd = 1
        if len(common_factors) > 0:
            gcd = common_factors[0]  # 最大公因数
        
        if len(common_factors) > 2:  # 有多个公因数（除了1和最大公因数）
            steps.append(CalculationStep(
                description=f"寻找公因数：{dividend} 和 {divisor} 的公因数有 {common_factors}",
                operation="寻找公因数",
                result=f"最大公因数是 {gcd}"
            ))
        else:
            steps.append(CalculationStep(
                description=f"计算最大公因数：gcd({dividend}, {divisor}) = {gcd}",
                operation="计算最大公因数",
                result=f"最大公因数是 {gcd}"
            ))
        
        # 展示约分过程
        new_dividend = dividend // gcd
        new_divisor = divisor // gcd
        
        steps.append(CalculationStep(
            description=f"同时除以最大公因数{gcd}进行约分",
            operation="约分操作",
            result=f"{dividend} ÷ {gcd} = {new_dividend}, {divisor} ÷ {gcd} = {new_divisor}"
        ))
        
        steps.append(CalculationStep(
            description=f"约分后：{dividend} ÷ {divisor} = {new_dividend} ÷ {new_divisor}",
            operation="约分结果",
            result=f"{new_dividend} ÷ {new_divisor}"
        ))
        
        # 计算最终结果
        if new_divisor == 1:
            final_result = new_dividend
            steps.append(CalculationStep(
                description=f"{new_dividend} ÷ 1 = {new_dividend}",
                operation="除以1",
                result=final_result
            ))
        else:
            final_result = new_dividend / new_divisor
            
            if isinstance(final_result, float) and final_result.is_integer():
                final_result = int(final_result)
                steps.append(CalculationStep(
                    description=f"{new_dividend} ÷ {new_divisor} = {final_result}",
                    operation="整除计算",
                    result=final_result
                ))
            else:
                steps.append(CalculationStep(
                    description=f"{new_dividend} ÷ {new_divisor} = {final_result}",
                    operation="除法计算",
                    result=final_result
                ))
        
        # 提供分数形式（如果适用）
        if new_divisor > 1 and new_divisor <= 20:
            steps.append(CalculationStep(
                description=f"分数形式：{dividend} ÷ {divisor} = {new_dividend}/{new_divisor}",
                operation="分数表示",
                result=f"最简分数：{new_dividend}/{new_divisor}"
            ))
        
        steps.append(CalculationStep(
            description=f"最终结果：{dividend} ÷ {divisor} = {final_result}",
            operation="确定结果",
            result=final_result,
            formula="约分法：(a×k) ÷ (b×k) = a ÷ b"
        ))
        
        # 提供验证
        verification = final_result * divisor
        if abs(verification - dividend) < 0.001:
            steps.append(CalculationStep(
                description=f"验证：{final_result} × {divisor} = {verification}",
                operation="验证正确",
                result="✓ 计算正确"
            ))
        
        return steps


"""
智能计算器引擎注册器
"""

from typing import List, Dict, Any, Optional
from .base_types import Formula, MathCalculator
from .formula_parser import FormulaParser


class SmartCalculatorReister:
    """智能计算器引擎 - 算法自动匹配机制"""
    
    def __init__(self):
        self.calculators: List[MathCalculator] = []
        self._register_default_calculators()
    
    def _register_default_calculators(self):
        """注册默认的算子"""
        # 导入具体的算子实现
        from .calculators.same_head_sum_ten_multiplication import SameHeadSumTenMultiplication
        from .calculators.head_sum_ten_tail_same_multiplication import HeadSumTenTailSameMultiplication
        from .calculators.ten_complement_addition import TenComplementAddition
        from .calculators.break_ten_subtraction import BreakTenSubtraction
        from .calculators.complement_addition_subtraction import ComplementAdditionSubtraction
        from .calculators.reversed_number_subtraction import ReversedNumberSubtraction
        from .calculators.multiply_by_nine import MultiplyByNine
        from .calculators.divide_by_five import DivideByFive
        from .calculators.multiply_by_eleven import MultiplyByEleven
        from .calculators.near_hundred_multiplication import NearHundredMultiplication
        from .calculators.near_hundreds_multiplication import NearHundredsMultiplication
        from .calculators.two_digit_square import TwoDigitSquare
        
        # 新增的算法
        from .calculators.multiply_by_twenty_five import MultiplyByTwentyFive
        from .calculators.divide_by_twenty_five import DivideByTwentyFive
        from .calculators.tens_one_multiplication import TensOneMultiplication
        from .calculators.reversed_number_addition import ReversedNumberAddition
        from .calculators.complement_ten_multiply_repeated import ComplementTenMultiplyRepeated
        from .calculators.hundred_plus_some_multiplication import HundredPlusSomeMultiplication
        from .calculators.hundred_plus_few_multiplication import HundredPlusFewMultiplication
        from .calculators.middle_number_multiplication import MiddleNumberMultiplication
        from .calculators.two_digit_subtraction import TwoDigitSubtraction
        
        # 通用乘法算法（最低优先级）
        from .calculators.general_multiplication import GeneralMultiplication
        
        # 新增的减法算法
        from .calculators.same_number_subtraction import SameNumberSubtraction
        from .calculators.consecutive_number_subtraction import ConsecutiveNumberSubtraction
        from .calculators.same_head_subtraction import SameHeadSubtraction
        from .calculators.tail_complement_ten_subtraction import TailComplementTenSubtraction
        from .calculators.near_hundred_subtraction import NearHundredSubtraction
        
        # 最新添加的减法算法
        from .calculators.split_subtraction import SplitSubtraction
        from .calculators.high_to_low_digit_subtraction import HighToLowDigitSubtraction
        from .calculators.reference_number_subtraction import ReferenceNumberSubtraction
        from .calculators.same_add_same_subtract import SameAddSameSubtract
        from .calculators.power_of_ten_subtraction import PowerOfTenSubtraction
        from .calculators.chain_subtraction_to_addition import ChainSubtractionToAddition
        from .calculators.remove_zero_subtraction import RemoveZeroSubtraction
        from .calculators.inverse_addition_subtraction import InverseAdditionSubtraction
        
        # 新增的加法算法
        from .calculators.same_number_addition import SameNumberAddition
        from .calculators.consecutive_number_addition import ConsecutiveNumberAddition
        from .calculators.same_head_addition import SameHeadAddition
        from .calculators.near_hundred_addition import NearHundredAddition
        from .calculators.reference_number_addition import ReferenceNumberAddition
        from .calculators.high_to_low_digit_addition import HighToLowDigitAddition
        from .calculators.segment_sum_addition import SegmentSumAddition
        from .calculators.rounding_addition import RoundingAddition
        from .calculators.split_addition import SplitAddition
        from .calculators.group_pairing_addition import GroupPairingAddition
        from .calculators.borrow_rounding_addition import BorrowRoundingAddition
        from .calculators.commutative_associative_addition import CommutativeAssociativeAddition
        
        # 新增的乘法算法
        from .calculators.multiply_by_powers_of_ten import MultiplyByPowersOfTen
        from .calculators.multiply_by_one_hundred_twenty_five import MultiplyByOneHundredTwentyFive
        from .calculators.multiply_by_twelve import MultiplyByTwelve
        from .calculators.multiply_by_fifteen import MultiplyByFifteen
        from .calculators.multiply_by_nineteen import MultiplyByNineteen
        from .calculators.difference_method_multiplication import DifferenceMethodMultiplication
        from .calculators.split_method_multiplication import SplitMethodMultiplication
        from .calculators.commutative_associative_multiplication import CommutativeAssociativeMultiplication
        
        # 新增的除法算法（按优先级重新排序）
        from .calculators.multiplication_table_division import MultiplicationTableDivision  # priority=7
        from .calculators.round_number_division import RoundNumberDivision  # priority=6
        from .calculators.near_round_division import NearRoundDivision  # priority=5
        from .calculators.split_division import SplitDivision  # priority=4
        from .calculators.integer_multiple_division import IntegerMultipleDivision  # priority=3
        from .calculators.general_division import GeneralDivision  # priority=1
        from .calculators.multiply_by_five import MultiplyByFive  # priority=4
        
        # 最新添加的除法算法
        from .calculators.power_of_ten_division import PowerOfTenDivision  # priority=8
        from .calculators.halving_division import HalvingDivision  # priority=7
        from .calculators.consecutive_halving_division import ConsecutiveHalvingDivision  # priority=7
        from .calculators.divide_by_one_hundred_twenty_five import DivideByOneHundredTwentyFive  # priority=6
        from .calculators.quotient_invariant_division import QuotientInvariantDivision  # priority=5
        from .calculators.reduction_division import ReductionDivision  # priority=6
        from .calculators.divide_by_nine import DivideByNine  # priority=6
        from .calculators.divide_by_eleven import DivideByEleven  # priority=6
        from .calculators.split_divisor_division import SplitDivisorDivision  # priority=5
        from .calculators.first_digit_estimate_division import FirstDigitEstimateDivision  # priority=4
        
        self.calculators = [
            MultiplyByPowersOfTen(),  # 优先级5：补零法（支持所有整零数）
            MultiplyByFive(),
            MultiplyByNine(),
            MultiplyByTwentyFive(),
            MultiplyByOneHundredTwentyFive(),  # 优先级3：乘125
            MultiplyByTwelve(),  # 优先级3：乘12
            MultiplyByFifteen(),  # 优先级3：乘15
            MultiplyByNineteen(),  # 优先级3：乘19
            DivideByFive(),  # 暂时禁用，解析器问题
            DivideByTwentyFive(),
            MultiplyByEleven(),
            TenComplementAddition(),
            SameNumberAddition(),
            ConsecutiveNumberAddition(), 
            RoundingAddition(),
            SameHeadAddition(),
            BorrowRoundingAddition(),
            ReferenceNumberAddition(),
            SegmentSumAddition(),
            HighToLowDigitAddition(),
            SplitAddition(),
            NearHundredAddition(),
            GroupPairingAddition(),
            CommutativeAssociativeAddition(),
            BreakTenSubtraction(),
            TwoDigitSubtraction(),
            SameNumberSubtraction(),
            ConsecutiveNumberSubtraction(),
            SameHeadSubtraction(),
            TailComplementTenSubtraction(),
            NearHundredSubtraction(),
            ComplementAdditionSubtraction(),
            ReversedNumberSubtraction(),
            # 最新添加的减法算法
            SplitSubtraction(),
            HighToLowDigitSubtraction(),
            ReferenceNumberSubtraction(),
            SameAddSameSubtract(),
            PowerOfTenSubtraction(),
            ChainSubtractionToAddition(),
            RemoveZeroSubtraction(),
            InverseAdditionSubtraction(),
            ReversedNumberAddition(),
            TensOneMultiplication(),
            ComplementTenMultiplyRepeated(),
            SameHeadSumTenMultiplication(),
            HeadSumTenTailSameMultiplication(),
            NearHundredsMultiplication(),  # 更通用的整百数乘法，优先级更高
            NearHundredMultiplication(),
            HundredPlusSomeMultiplication(),
            HundredPlusFewMultiplication(),  # 几百零几乘法
            MiddleNumberMultiplication(),
            TwoDigitSquare(),
            DifferenceMethodMultiplication(),  # 优先级6：差额法
            CommutativeAssociativeMultiplication(),  # 优先级5：交换结合法
            SplitMethodMultiplication(),  # 优先级4：分拆法
            GeneralMultiplication(),  # 通用乘法算法，最低优先级
            # 除法算法（按优先级排序：8→7→6→5→4→3→1）
            PowerOfTenDivision(),          # 优先级8：补零法除法
            MultiplicationTableDivision(), # 优先级7：九九表速算
            HalvingDivision(),             # 优先级7：减半法
            ConsecutiveHalvingDivision(),  # 优先级7：连续减半法
            RoundNumberDivision(),          # 优先级6：整十除法
            DivideByOneHundredTwentyFive(), # 优先级6：除以125
            ReductionDivision(),           # 优先级6：约分法
            DivideByNine(),                # 优先级6：除以9
            DivideByEleven(),              # 优先级6：除以11
            NearRoundDivision(),           # 优先级5：近整十除法
            QuotientInvariantDivision(),   # 优先级5：商不变性质
            SplitDivisorDivision(),        # 优先级5：拆分除数法
            SplitDivision(),               # 优先级4：拆分除法
            FirstDigitEstimateDivision(),  # 优先级4：首数估商法
            IntegerMultipleDivision(),      # 优先级3：整数倍数除法
            GeneralDivision(),             # 优先级1：通用除法（兜底）
        ]
    
    def get_division_remainder(self):
        from .calculators.division_with_remainder import DivisionWithRemainder  # priority=4
        """设置除法计算模式"""
        return DivisionWithRemainder()
    def get_revert_calculator(self) -> MathCalculator:
        """获取逆运算算子实例"""
        from .calculators.revert_calculator import RevertCalculator
        return RevertCalculator()
    def register_calculator(self, calculator: MathCalculator):
        """注册新的算子"""
        self.calculators.append(calculator)
    
    def find_matching_calculators(self, formula: Formula) -> List[MathCalculator]:
        """找到所有匹配的算子候选集合"""
        matching = []
        for calc in self.calculators:
            if calc.is_match_pattern(formula):
                matching.append(calc)
        return matching



"""
混合运算算子注册器
"""

from typing import List, Dict, Any, Optional
from .base_types import MixedCalculator, MixedProblem


class MixedCalculatorRegistry:
    """混合运算算子注册器"""
    
    def __init__(self):
        self.calculators: List[MixedCalculator] = []
        self._register_default_calculators()
    
    def _register_default_calculators(self):
        """注册默认的混合运算算子"""
        from .calculators.parentheses_evaluator import ParenthesesEvaluator
        from .calculators.zero_one_optimizer import ZeroOneOptimizer
        from .calculators.mul_div_left_to_right import MulDivLeftToRightCalculator
        from .calculators.add_sub_left_to_right import AddSubLeftToRightCalculator
        from .calculators.standard_order_evaluator import StandardOrderEvaluator
        from .calculators.bracketed_mix_evaluator import BracketedMixEvaluator
        from .calculators.distribution_optimizer import DistributionOptimizer
        from .calculators.complement_rewriting import ComplementRewritingCalculator
        
        # 按优先级注册算子（优先级数字越小越优先）
        calculators = [
            ParenthesesEvaluator(),        # 1 - 括号优先
            BracketedMixEvaluator(),       # 2 - 含括号混合运算
            ZeroOneOptimizer(),            # 3 - 0/1优化（仅纯乘除）
            MulDivLeftToRightCalculator(), # 5 - 乘除链求值
            DistributionOptimizer(),       # 6 - 分配律优化
            ComplementRewritingCalculator(), # 7 - 凑整/补数重写
            AddSubLeftToRightCalculator(), # 8 - 加减链
            StandardOrderEvaluator(),      # 10 - 标准无括号混合求值（最后兜底）
        ]
        
        for calc in calculators:
            self.register(calc)
    
    def register(self, calculator: MixedCalculator):
        """注册算子"""
        self.calculators.append(calculator)
        # 按优先级排序（优先级数字小的在前）
        self.calculators.sort(key=lambda x: x.priority)
    
    def find_matching_calculators(self, problem: MixedProblem) -> List[MixedCalculator]:
        """找到匹配的算子"""
        matching = []
        
        for calculator in self.calculators:
            match_result = calculator.is_match_pattern(problem)
            if match_result.get("matched", False):
                matching.append(calculator)
        
        return matching
    
    def get_best_calculator(self, problem: MixedProblem) -> Optional[MixedCalculator]:
        """获取最佳算子"""
        matching = self.find_matching_calculators(problem)
        
        if not matching:
            return None
        
        # 按优先级和匹配分数排序
        matching.sort(key=lambda x: (x.priority, -x.is_match_pattern(problem).get("score", 0)))
        
        return matching[0]
    
    def get_all_calculators(self) -> List[MixedCalculator]:
        """获取所有注册的算子"""
        return self.calculators.copy()
    
    def get_calculator_by_name(self, name: str) -> Optional[MixedCalculator]:
        """根据名称获取算子"""
        for calc in self.calculators:
            if calc.name == name:
                return calc
        return None



"""
分数比较算子 - 比较分数大小
"""

from typing import Dict, Any
from fractions import Fraction
from ..base_types import FractionCalculator, FractionProblem, FractionResult, FractionStep, OperationType


class FractionCompareCalculator(FractionCalculator):
    """分数比较算子"""
    
    def __init__(self):
        super().__init__("分数比较", "比较分数大小", priority=3)
    
    def is_match_pattern(self, problem: FractionProblem) -> Dict[str, Any]:
        """匹配分数比较问题"""
        if problem.operation != OperationType.COMPARE:
            return {"matched": False, "score": 0.0, "reason": "不是比较"}
        
        if len(problem.operands) != 2:
            return {"matched": False, "score": 0.0, "reason": "需要两个操作数"}
        
        return {"matched": True, "score": 1.0, "reason": "分数比较"}
    
    def solve(self, problem: FractionProblem) -> FractionResult:
        """执行分数比较"""
        try:
            operand1, operand2 = problem.operands
            frac1, frac2 = operand1.fraction, operand2.fraction
            steps = []
            
            # 步骤1：检查快速路径
            if frac1.denominator == frac2.denominator:
                # 同分母比较
                steps.append(FractionStep(
                    description=f"同分母分数比较：{frac1} 与 {frac2}，分母都是{frac1.denominator}",
                    operation="同分母分数比较",
                    result="直接比较分子大小"
                ))
                
                if frac1.numerator > frac2.numerator:
                    comparison = ">"
                elif frac1.numerator < frac2.numerator:
                    comparison = "<"
                else:
                    comparison = "="
                
                steps.append(FractionStep(
                    description=f"比较分子：{frac1.numerator} {comparison} {frac2.numerator}",
                    operation="比较分子",
                    result=f"{frac1} {comparison} {frac2}"
                ))
                
            elif frac1.numerator == frac2.numerator:
                # 同分子比较
                steps.append(FractionStep(
                    description=f"同分子分数比较：{frac1} 与 {frac2}，分子都是{frac1.numerator}",
                    operation="同分子分数比较",
                    result="分母小者更大"
                ))
                
                if frac1.denominator < frac2.denominator:
                    comparison = ">"
                elif frac1.denominator > frac2.denominator:
                    comparison = "<"
                else:
                    comparison = "="
                
                steps.append(FractionStep(
                    description=f"比较分母：{frac1.denominator} vs {frac2.denominator}，分母小的分数大",
                    operation="比较分母",
                    result=f"{frac1} {comparison} {frac2}"
                ))
                
            else:
                # 交叉相乘比较
                cross1 = frac1.numerator * frac2.denominator
                cross2 = frac2.numerator * frac1.denominator
                
                steps.append(FractionStep(
                    description=f"交叉相乘比较：{frac1.numerator}×{frac2.denominator} 与 {frac2.numerator}×{frac1.denominator}",
                    operation="交叉相乘比较",
                    result=f"{cross1} 与 {cross2}",
                    formula="a/b ? c/d ⟺ a×d ? c×b"
                ))
                
                if cross1 > cross2:
                    comparison = ">"
                elif cross1 < cross2:
                    comparison = "<"
                else:
                    comparison = "="
                
                steps.append(FractionStep(
                    description=f"比较结果：{cross1} {comparison} {cross2}",
                    operation="比较结果",
                    result=f"{frac1} {comparison} {frac2}"
                ))
            
            return FractionResult(
                success=True,
                result=comparison,
                steps=steps,
                step_count=len(steps),
                formula=f"分数比较：{frac1} {comparison} {frac2}",
                validation=True
            )
            
        except Exception as e:
            return FractionResult(
                success=False,
                error=f"分数比较失败: {str(e)}"
            )



"""
时间单位换算算子
"""

from typing import List, Dict
from decimal import Decimal, getcontext
from ..base_types import UnitCalculator, UnitProblem, UnitType, UnitValue, ConversionStep
import datetime

# 设置高精度计算
getcontext().prec = 28

class TimeUnitCalculator(UnitCalculator):
    """时间单位换算算子"""
    
    def __init__(self):
        super().__init__("时间单位换算", "处理年、月、日、时、分、秒之间的换算", UnitType.TIME)
    
    def _initialize_conversion_rules(self) -> Dict[str, Dict[str, float]]:
        """初始化时间单位换算规则"""
        return {
            # 年换算（使用平均值，特殊情况在solve方法中处理）
            "年": {"年": 1, "month": 12, "月": 12, "日": 365.25, "天": 365.25, "时": 365.25*24, "小时": 365.25*24, "h": 365.25*24, "分": 365.25*24*60, "秒": 365.25*24*3600},
            "year": {"年": 1, "year": 1, "month": 12, "月": 12, "日": 365.25, "天": 365.25, "时": 365.25*24, "小时": 365.25*24, "h": 365.25*24},
            
            # 月换算（使用平均值，特殊情况在solve方法中处理）
            "月": {"年": 1/12, "year": 1/12, "月": 1, "month": 1, "日": 30.44, "天": 30.44, "时": 30.44*24, "小时": 30.44*24, "h": 30.44*24, "分": 30.44*24*60, "秒": 30.44*24*3600},
            "month": {"年": 1/12, "year": 1/12, "月": 1, "month": 1, "日": 30.44, "天": 30.44, "时": 30.44*24, "小时": 30.44*24, "h": 30.44*24},
            
            # 秒换算
            "s": {"s": 1, "min": 1/60, "h": 1/3600, "分": 1/60, "时": 1/3600, "小时": 1/3600, "秒": 1, "日": 1/(24*3600), "天": 1/(24*3600)},
            "秒": {"秒": 1, "分": 1/60, "时": 1/3600, "小时": 1/3600, "s": 1, "min": 1/60, "h": 1/3600, "日": 1/(24*3600), "天": 1/(24*3600)},
            
            # 分钟换算
            "min": {"s": 60, "min": 1, "h": 1/60, "秒": 60, "分": 1, "时": 1/60, "小时": 1/60, "日": 1/(24*60), "天": 1/(24*60)},
            "分": {"秒": 60, "分": 1, "时": 1/60, "小时": 1/60, "s": 60, "min": 1, "h": 1/60, "日": 1/(24*60), "天": 1/(24*60)},
            "分钟": {"秒": 60, "分钟": 1, "小时": 1/60, "s": 60, "min": 1, "h": 1/60, "分": 1, "时": 1/60, "日": 1/(24*60), "天": 1/(24*60)},
            
            # 小时换算
            "h": {"s": 3600, "min": 60, "h": 1, "秒": 3600, "分": 60, "时": 1, "小时": 1, "日": 1/24, "天": 1/24},
            "时": {"秒": 3600, "分": 60, "时": 1, "日": 1/24, "天": 1/24, "s": 3600, "min": 60, "h": 1, "小时": 1},
            "小时": {"秒": 3600, "分钟": 60, "小时": 1, "日": 1/24, "天": 1/24, "s": 3600, "min": 60, "h": 1, "分": 60, "时": 1},
            
            # 日换算
            "日": {"小时": 24, "时": 24, "h": 24, "日": 1, "天": 1, "分": 24*60, "秒": 24*3600, "月": 1/30.44, "month": 1/30.44, "年": 1/365.25, "year": 1/365.25},
            "天": {"小时": 24, "时": 24, "h": 24, "日": 1, "天": 1, "分": 24*60, "秒": 24*3600, "月": 1/30.44, "month": 1/30.44, "年": 1/365.25, "year": 1/365.25},
            "day": {"小时": 24, "时": 24, "h": 24, "日": 1, "天": 1, "day": 1},
        }
    
    def _is_leap_year(self, year: int) -> bool:
        """判断是否为闰年"""
        return (year % 4 == 0 and year % 100 != 0) or (year % 400 == 0)
    
    def _get_days_in_month(self, year: int, month: int) -> int:
        """获取指定年月的天数"""
        if month in [1, 3, 5, 7, 8, 10, 12]:
            return 31
        elif month in [4, 6, 9, 11]:
            return 30
        elif month == 2:
            return 29 if self._is_leap_year(year) else 28
        else:
            raise ValueError(f"无效的月份：{month}")
    
    def _get_days_in_year(self, year: int) -> int:
        """获取指定年份的天数"""
        return 366 if self._is_leap_year(year) else 365
    
    def parse_year_month(self, problem) -> UnitProblem:
        new_source_values = []
        for v in problem.source_values:
            if v.unit == "年":
                new_source_values.append(UnitValue(v.value * 12, "月"))
            else:
                new_source_values.append(v)
        problem = UnitProblem(
            original_text=problem.original_text,
            unit_type=problem.unit_type,
            source_values=new_source_values,
            target_unit=problem.target_unit
        )
        return problem
    
    def _handle_year_month_conversion(self, problem: UnitProblem) -> tuple[float, List[ConversionStep]]:
        """处理涉及年、月的特殊换算"""
        steps = []
        
        # 检查是否是特定年月到天数的转换
        source_units = [v.unit for v in problem.source_values]
        target_unit = problem.target_unit
        
        if len(problem.source_values) == 2 and "年" in source_units and "月" in source_units and target_unit in ["天", "日"]:
            # xx年x月有多少天的情况
            year_value = None
            month_value = None
            
            for source_value in problem.source_values:
                if source_value.unit == "年":
                    year_value = int(source_value.value)
                elif source_value.unit == "月":
                    month_value = int(source_value.value)
            
            if not year_value or year_value < 1000:
                return -1, []
            
            if year_value and month_value:
                days = self._get_days_in_month(year_value, month_value)
                
                steps.append(ConversionStep(
                    description=f"查找{year_value}年{month_value}月的天数",
                    operation="年月天数查询",
                    from_value=UnitValue(f"{year_value}年{month_value}月", ""),
                    result=UnitValue(days, target_unit),
                    formula=f"{year_value}年{month_value}月 = {days}天"
                ))
                
                # 添加闰年说明
                if month_value == 2:
                    leap_status = "闰年" if self._is_leap_year(year_value) else "平年"
                    steps.append(ConversionStep(
                        description=f"{year_value}年是{leap_status}，2月有{days}天",
                        operation="闰年判断",
                        from_value=UnitValue(year_value, "年"),
                        result=UnitValue(days, "天"),
                        formula=f"{leap_status}2月 = {days}天"
                    ))
                
                return float(days), steps
        
        # 其他年月换算使用标准流程
        return None, []

    def is_match_pattern(self, problem: UnitProblem) -> bool:
        """判断是否为时间单位换算问题"""
        return problem.unit_type == UnitType.TIME
    
    def solve(self, problem: UnitProblem) -> tuple[float, List[ConversionStep]]:
        """求解时间单位换算问题，支持 operator（+/-）单位运算"""
        # 先尝试特殊的年月换算
        special_result, special_steps = self._handle_year_month_conversion(problem)
        if special_result and len(special_steps) > 0:
            return special_result, special_steps
        elif special_result == -1:
            problem = self.parse_year_month(problem)
        steps = []
        total_result = Decimal('0')
        operator = "+"
        for source_value in problem.source_values:
            conversion_factor = self.get_conversion_factor(source_value.unit, problem.target_unit)
            if conversion_factor is None:
                raise ValueError(f"无法从{source_value.unit}换算到{problem.target_unit}")
            source_decimal = Decimal(str(source_value.value))
            factor_decimal = Decimal(str(conversion_factor))
            converted_decimal = source_decimal * factor_decimal
            converted_value = float(converted_decimal)
            if operator == '-':
                total_result -= converted_decimal
            elif operator == '+':
                total_result += converted_decimal
            operator = getattr(source_value, 'operator', '+')
            formula = self._get_standard_formula(source_value.unit, problem.target_unit)
            description = f"换算{source_value.unit}到{problem.target_unit}"
            result = UnitValue(converted_value, problem.target_unit)
            operation = "时间单位换算"
            if source_value.unit == problem.target_unit:
                description = f"{result}"
                operation = f"记录“{source_value.unit}”单位数值"
            step = ConversionStep(
                description=description,
                operation=operation,
                from_value=source_value,
                result=result,
                formula=formula
            )
            steps.append(step)
        if len(problem.source_values) > 1:
            formula_parts = []
            for v in problem.source_values:
                factor = self.get_conversion_factor(v.unit, problem.target_unit)
                if factor is not None:
                    formula_parts.append(str(v.value * factor) + getattr(v, 'operator', '+'))
                else:
                    formula_parts.append(str(v.value) + getattr(v, 'operator', '+'))
            formula = ' '.join(formula_parts).rstrip('+')
            step = ConversionStep(
                description=f"合并时间换算结果",
                operation="时间求和",
                from_value=UnitValue(0, problem.target_unit),
                result=UnitValue(float(total_result), problem.target_unit),
                formula=f"总时间 = {formula}"
            )
            steps.append(step)
        final_result = self._round_result(float(total_result))
        return final_result, steps
    
    def _get_standard_formula(self, from_unit: str, to_unit: str) -> str:
        """获取标准化的换算公式（大单位 = 小单位倍数）"""
        # 定义单位大小顺序（从大到小）
        unit_hierarchy = {
            "年": 6, "year": 6,
            "月": 5, "month": 5,
            "天": 4, "日": 4, "day": 4,
            "时": 3, "小时": 3, "h": 3,
            "分": 2, "分钟": 2, "min": 2,
            "秒": 1, "s": 1
        }

        # 如果单位相同，直接返回
        if from_unit == to_unit:
            return ""
        
        from_level = unit_hierarchy.get(from_unit, 0)
        to_level = unit_hierarchy.get(to_unit, 0)
        
        # 确定哪个是大单位，哪个是小单位
        if from_level > to_level:
            # 从大单位换到小单位
            big_unit, small_unit = from_unit, to_unit
            factor = self.get_conversion_factor(from_unit, to_unit)
        else:
            # 从小单位换到大单位
            big_unit, small_unit = to_unit, from_unit
            factor = 1 / self.get_conversion_factor(from_unit, to_unit) if self.get_conversion_factor(from_unit, to_unit) else 1
        
        # 生成标准公式
        if factor and factor != 1:
            if factor == int(factor):
                return f"1{big_unit} = {int(factor)}{small_unit}"
            else:
                # 对于不是整数的换算，保留合理的小数位数
                if factor < 1:
                    return f"1{small_unit} = {1/factor:.6g}{big_unit}"
                else:
                    return f"1{big_unit} = {factor:.6g}{small_unit}"
        else:
            return f"1{from_unit} = 1{to_unit}"



"""
分数乘整数算子
"""

from typing import Dict, Any
from fractions import Fraction
from math import gcd
from ..base_types import FractionCalculator, FractionProblem, FractionResult, FractionStep, OperationType, FractionType


class FractionIntegerMultiplyCalculator(FractionCalculator):
    """分数乘整数算子"""
    
    def __init__(self):
        super().__init__("分数乘整数", "分数与整数相乘，优先约分", priority=5)
    
    def is_match_pattern(self, problem: FractionProblem) -> Dict[str, Any]:
        """匹配分数乘整数问题"""
        if problem.operation != OperationType.MULTIPLY:
            return {"matched": False, "score": 0.0, "reason": "不是乘法"}
        
        if len(problem.operands) != 2:
            return {"matched": False, "score": 0.0, "reason": "需要两个操作数"}
        
        # 检查是否有一个分数和一个整数
        op1, op2 = problem.operands
        
        # 检查第一个是分数，第二个是整数
        is_frac_int_1 = (op1.fraction_type in [FractionType.PROPER, FractionType.IMPROPER, FractionType.MIXED] and 
                        op2.fraction_type == FractionType.INTEGER)
        
        # 检查第一个是整数，第二个是分数
        is_int_frac_1 = (op1.fraction_type == FractionType.INTEGER and 
                        op2.fraction_type in [FractionType.PROPER, FractionType.IMPROPER, FractionType.MIXED])
        
        # 额外检查：确保整数的分母确实为1
        if is_frac_int_1:
            is_really_integer = op2.fraction.denominator == 1
        elif is_int_frac_1:
            is_really_integer = op1.fraction.denominator == 1
        else:
            is_really_integer = False
        
        if (is_frac_int_1 or is_int_frac_1) and is_really_integer:
            return {"matched": True, "score": 1.0, "reason": "分数乘整数"}
        else:
            return {"matched": False, "score": 0.0, "reason": "不是分数乘整数"}
    
    def solve(self, problem: FractionProblem) -> FractionResult:
        """执行分数乘整数"""
        try:
            op1, op2 = problem.operands
            steps = []
            
            # 确定哪个是分数，哪个是整数
            if op1.fraction_type == FractionType.INTEGER:
                integer_val = int(op1.fraction)
                fraction_op = op2
                fraction_val = op2.fraction
            else:
                integer_val = int(op2.fraction)
                fraction_op = op1
                fraction_val = op1.fraction
            
            # 步骤1：处理带分数
            if fraction_op.is_mixed:
                improper = Fraction(fraction_op.whole_part * fraction_val.denominator + fraction_val.numerator,
                                  fraction_val.denominator)
                steps.append(FractionStep(
                    description=f"带分数转假分数：{fraction_op.raw} → {improper}",
                    operation="转为假分数",
                    result=str(improper)
                ))
                fraction_val = improper
            
            # 步骤2：尝试约分整数与分母
            original_int = integer_val
            original_denom = fraction_val.denominator
            
            common_factor = gcd(abs(integer_val), fraction_val.denominator)
            if common_factor > 1:
                integer_val //= common_factor
                new_denom = fraction_val.denominator // common_factor
                
                steps.append(FractionStep(
                    description=f"约分整数与分母：整数{original_int}与分母{original_denom}的公因子{common_factor}",
                    operation="约分整数与分母",
                    result=f"约分后：整数={integer_val}，分母={new_denom}"
                ))
            else:
                new_denom = fraction_val.denominator
                steps.append(FractionStep(
                    description=f"整数{integer_val}与分母{fraction_val.denominator}无公因子，直接相乘",
                    operation="判断是否约分",
                    result="无需约分"
                ))
            
            # 步骤3：分子乘以整数
            new_numerator = fraction_val.numerator * integer_val
            result_fraction = Fraction(new_numerator, new_denom)
            
            steps.append(FractionStep(
                description=f"分子乘以整数：{fraction_val.numerator} × {integer_val} = {new_numerator}",
                operation="分子乘以整数",
                result=f"{new_numerator}/{new_denom}",
                formula=f"(a/b)*n = (a*n)/b"
            ))
            
            # 步骤4：最终约分
            final_result = result_fraction
            if result_fraction.denominator != 1:
                final_gcd = gcd(abs(result_fraction.numerator), result_fraction.denominator)
                if final_gcd > 1:
                    final_result = Fraction(result_fraction.numerator // final_gcd,
                                          result_fraction.denominator // final_gcd)
                    steps.append(FractionStep(
                        description=f"最终约分：{result_fraction} → {final_result}",
                        operation="取得最简分数",
                        result=str(final_result)
                    ))
            
            # 步骤5：根据需要转为带分数
            display_result = final_result
            if (problem.display_mixed and 
                abs(final_result.numerator) >= final_result.denominator and 
                final_result.denominator != 1):
                
                whole = final_result.numerator // final_result.denominator
                remainder = abs(final_result.numerator) % final_result.denominator
                
                if remainder == 0:
                    display_result = str(whole)
                else:
                    if final_result.numerator < 0:
                        display_result = f"-{abs(whole)} {remainder}/{final_result.denominator}"
                    else:
                        display_result = f"{whole} {remainder}/{final_result.denominator}"
                
                steps.append(FractionStep(
                    description=f"转为带分数：{final_result} = {display_result}",
                    operation="转为带分数",
                    result=str(display_result)
                ))
            
            return FractionResult(
                success=True,
                result=display_result,
                steps=steps,
                step_count=len(steps),
                formula=f"分数乘整数：{fraction_op.raw} × {original_int} = {display_result}",
                validation=True
            )
            
        except Exception as e:
            return FractionResult(
                success=False,
                error=f"分数乘整数计算失败: {str(e)}"
            )



"""
近整百加法实现
处理接近整百数的加法，如198+207=405
"""

from typing import List
from ..base_types import MathCalculator, Formula, CalculationStep


class NearHundredAddition(MathCalculator):
    """近整百加法算法"""
    
    def __init__(self):
        super().__init__("近整百加法", "处理接近整百数的加法", priority=5)
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：两个数都接近整百数"""
        if formula.type != "addition":
            return False
        
        numbers = formula.get_numbers()
        if len(numbers) != 2:
            return False
        
        try:
            addend1, addend2 = [elem.get_numeric_value() for elem in numbers]
            if not (isinstance(addend1, int) and isinstance(addend2, int)):
                return False
            
            # 三位数且都接近整百数
            if not (100 <= addend1 <= 999 and 100 <= addend2 <= 999):
                return False
            
            # 检查是否都接近整百数（与最近整百数的差距小于等于15）
            addend1_hundred = round(addend1 / 100) * 100
            addend2_hundred = round(addend2 / 100) * 100
            
            addend1_diff = abs(addend1 - addend1_hundred)
            addend2_diff = abs(addend2 - addend2_hundred)
            
            # 都接近整百数
            return addend1_diff <= 15 and addend2_diff <= 15
        except:
            return False
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建近整百加法步骤"""
        numbers = formula.get_numbers()
        addend1 = int(numbers[0].get_numeric_value())
        addend2 = int(numbers[1].get_numeric_value())
        
        steps = []
        
        steps.append(CalculationStep(
            description=f"{addend1} + {addend2} 使用近整百加法",
            operation="识别近整百数",
            result="两个加数都接近整百数"
        ))
        
        # 找到最近的整百数
        addend1_hundred = round(addend1 / 100) * 100
        addend2_hundred = round(addend2 / 100) * 100
        
        # 计算与整百数的差值
        addend1_diff = addend1 - addend1_hundred
        addend2_diff = addend2 - addend2_hundred
        
        steps.append(CalculationStep(
            description=f"分解为整百数部分：{addend1} = {addend1_hundred} + ({addend1_diff}), {addend2} = {addend2_hundred} + ({addend2_diff})",
            operation="分解整百数",
            result=f"整百数部分：{addend1_hundred} + {addend2_hundred}"
        ))
        
        # 先计算整百数部分
        hundred_sum = addend1_hundred + addend2_hundred
        steps.append(CalculationStep(
            description=f"整百数相加：{addend1_hundred} + {addend2_hundred} = {hundred_sum}",
            operation="计算整百数和",
            result=hundred_sum
        ))
        
        # 计算余数部分
        remainder_sum = addend1_diff + addend2_diff
        steps.append(CalculationStep(
            description=f"余数相加：({addend1_diff}) + ({addend2_diff}) = {remainder_sum}",
            operation="计算余数和",
            result=remainder_sum
        ))
        
        # 最终结果
        final_result = hundred_sum + remainder_sum
        steps.append(CalculationStep(
            description=f"组合结果：{hundred_sum} + ({remainder_sum}) = {final_result}",
            operation="合并结果",
            result=final_result,
            formula="近整百加法：(100a+x) + (100b+y) = 100(a+b) + (x+y)"
        ))
        
        return steps


"""
小数乘法整化法算子
去小数点乘后再补位的技巧
"""

from typing import List
from .decima_calculator import DecimaCalculator
from ..base_types import Formula, CalculationStep


class DecimalIntegerMethod(DecimaCalculator):
    """小数乘法整化法算子"""
    
    def __init__(self):
        super().__init__("小数整化乘法", "去小数点乘后再补位", priority=7)
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：小数乘法"""
        if not super().is_match_pattern(formula):
            return False
        return formula.type == "multiplication"
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建整化乘法步骤"""
        numbers = formula.get_numbers()
        a, b = [elem.get_numeric_value() for elem in numbers]
        
        steps = []
        
        # 分析原始数据
        a_str, b_str = str(a), str(b)
        a_places = len(a_str.split('.')[1]) if '.' in a_str else 0
        b_places = len(b_str.split('.')[1]) if '.' in b_str else 0
        total_places = a_places + b_places
        
        steps.append(CalculationStep(
            description=f"分析：{a}({a_places}位小数) × {b}({b_places}位小数)",
            operation="分析小数位数",
            result=f"结果需要{total_places}位小数"
        ))
        
        # 去小数点转为整数
        a_int = int(a_str.replace('.', ''))
        b_int = int(b_str.replace('.', ''))
        
        steps.append(CalculationStep(
            description=f"去小数点：{a} × {b} → {a_int} × {b_int}",
            operation="转换为整数",
            result=f"{a_int} × {b_int}"
        ))
        
        # 整数相乘
        product_int = a_int * b_int
        steps.append(CalculationStep(
            description=f"整数相乘：{a_int} × {b_int} = {product_int}",
            operation="整数乘法",
            result=product_int
        ))
        
        # 恢复小数点
        final_result = product_int / (10 ** total_places)
        decimal_point_pos = len(str(product_int)) - total_places
        
        steps.append(CalculationStep(
            description=f"恢复小数点：从右数{total_places}位 → {final_result}",
            operation="恢复小数点",
            result=final_result,
            formula="整化法：去小数点 → 整数相乘 → 移动小数点"
        ))
        
        return steps



"""
整数与分数比较算子 - 比较整数和分数的大小
"""

from typing import Dict, Any
from fractions import Fraction
from ..base_types import FractionCalculator, FractionProblem, FractionResult, FractionStep, OperationType, to_latex


class IntegerFractionCompareCalculator(FractionCalculator):
    """整数与分数比较算子"""
    def __init__(self):
        super().__init__("整数与分数比较", "比较整数和分数的大小", priority=5)

    def is_match_pattern(self, problem: FractionProblem) -> Dict[str, Any]:
        """匹配整数与分数比较问题"""
        if problem.operation != OperationType.COMPARE:
            return {"matched": False, "score": 0.0, "reason": "不是比较"}
        if len(problem.operands) != 2:
            return {"matched": False, "score": 0.0, "reason": "需要两个操作数"}
        op1 = problem.operands[0]
        if isinstance(op1.fraction, Fraction) and not op1.fraction.is_integer:
            return {"matched": False, "score": 0.0, "reason": "不是整数与分数比较"}
        return {"matched": True, "score": 5.0, "reason": "整数与分数比较"}

    def solve(self, problem: FractionProblem) -> FractionResult:
        """执行整数与分数比较，保证比较方向与表达式顺序一致"""
        try:
            op1, op2 = problem.operands
            # 判断左右两侧哪个是整数，哪个是分数
            left, right = op1, op2
            left_str = str(left.fraction)
            right_str = to_latex(right.fraction)
            # 标记左右类型
            left_is_int = True if op2.original_format == 'latex' else False
            # 统一将整数和分数都转为 Fraction
            left_frac = left.fraction
            right_frac = right.fraction
            steps = []
            # 整数转分数步骤
            if left_is_int:
                steps.append(FractionStep(
                    description=f"将整数 {left_str} 转为分数 {to_latex(left_frac)}",
                    operation="整数转分数",
                    result=str(left_frac)
                ))
            else:
                steps.append(FractionStep(
                    description=f"将整数 {right_str} 转为分数 {to_latex(right_frac)}",
                    operation="整数转分数",
                    result=str(right_frac)
                ))
            # 交叉相乘
            cross_left = left_frac.numerator * right_frac.denominator
            cross_right = right_frac.numerator * left_frac.denominator
            steps.append(FractionStep(
                description=f"交叉相乘比较：{left_frac.numerator}×{right_frac.denominator} 与 {right_frac.numerator}×{left_frac.denominator}",
                operation="cross_multiply_compare",
                result=f"{cross_left} 与 {cross_right}",
                formula="a/b ? c/d ⟺ a×d ? c×b"
            ))
            if cross_left > cross_right:
                comparison = ">"
            elif cross_left < cross_right:
                comparison = "<"
            else:
                comparison = "="
            steps.append(FractionStep(
                description=f"比较结果：{cross_left} {comparison} {cross_right}",
                operation="比较结果",
                result=f"{left_str} {comparison} {right_str}"
            ))
            return FractionResult(
                success=True,
                result=comparison,
                steps=steps,
                name="整数与分数比较",
                step_count=len(steps),
                formula=f"整数与分数比较：{left_str} {comparison} {right_str}",
                validation=True
            )
        except Exception as e:
            return FractionResult(
                success=False,
                error=f"整数与分数比较失败: {str(e)}"
            )



"""
标准无括号混合求值算子 - 按运算优先级处理混合运算
"""

import re
from typing import Dict, Any, List, Tuple
from ..base_types import MixedCalculator, MixedProblem, MixedResult, MixedStep


class StandardOrderEvaluator(MixedCalculator):
    """标准无括号混合求值算子"""
    
    def __init__(self):
        super().__init__("标准混合运算", "按运算优先级处理无括号混合运算", priority=10)
    
    def is_match_pattern(self, problem: MixedProblem) -> Dict[str, Any]:
        """匹配包含混合运算的无括号表达式"""
        has_multiply_divide = any(op in ['*', '/'] for op in problem.operations)
        has_add_subtract = any(op in ['+', '-'] for op in problem.operations if op not in ['(', ')'])
        # 只有同时有乘除和加减才算混合运算
        if (has_multiply_divide or has_add_subtract) and not problem.has_parentheses:
            return {
                "matched": True,
                "score": 0.8,
                "reason": "无括号混合运算，需按优先级处理"
            }
        return {"matched": False, "score": 0.0, "reason": "不是混合运算或包含括号"}
    
    def solve(self, problem: MixedProblem) -> MixedResult:
        """执行标准混合运算"""
        try:
            expression = problem.expression
            steps = []
            
            steps.append(MixedStep(
                description="开始混合运算",
                operation="识别优先级",
                inputs=[expression],
                result="识别运算优先级",
                formula="乘除优先级 > 加减优先级"
            ))
            
            # 使用Python的eval来确保正确的运算优先级，但记录步骤
            result_value = self._evaluate_with_steps(expression, steps)
            
            steps.append(MixedStep(
                description="混合运算完成",
                operation="计算完成",
                inputs=[expression],
                result=str(result_value),
                formula=f"{expression} = {result_value}"
            ))
            
            return MixedResult(
                success=True,
                result=result_value,
                steps=steps,
                step_count=len(steps),
                formula=f"混合运算: {expression} = {result_value}",
                validation=True,
                technique_used="运算优先级"
            )
            
        except Exception as e:
            return MixedResult(
                success=False,
                error=f"标准混合运算失败: {str(e)}"
            )
    
    def _evaluate_with_steps(self, expression: str, steps: List[MixedStep]) -> float:
        """按步骤计算表达式，确保正确的运算优先级"""
        
        # 先找到所有乘除运算并逐个处理
        current_expr = expression
        
        # 使用正则表达式匹配乘除运算（包括负数）
        mul_div_pattern = r'(-?\d*\.?\d+)\s*([*/])\s*(-?\d*\.?\d+)'
        
        while re.search(mul_div_pattern, current_expr):
            match = re.search(mul_div_pattern, current_expr)
            if not match:
                break
            
            full_match = match.group(0)
            left_operand = float(match.group(1))
            operator = match.group(2)
            right_operand = float(match.group(3))
            
            # 执行运算
            if operator == '*':
                result = left_operand * right_operand
                op_desc = "乘法"
                op_symbol = "×"
            else:  # operator == '/'
                if right_operand == 0:
                    raise ValueError("除数不能为0")
                result = left_operand / right_operand
                op_desc = "除法"
                op_symbol = "÷"
            
            # 格式化结果
            if isinstance(result, float) and result.is_integer():
                result = int(result)
            
            steps.append(MixedStep(
                description=f"先算{op_desc}",
                operation=f"先算{op_desc}",
                inputs=[str(left_operand), str(right_operand)],
                result=str(result),
                formula=f"{left_operand} {op_symbol} {right_operand} = {result}"
            ))
            
            # 替换表达式中的这部分
            current_expr = current_expr.replace(full_match, str(result), 1)
        
        # 现在处理剩余的加减运算
        if re.search(r'[+\-]', current_expr):
            steps.append(MixedStep(
                description="继续处理加减",
                operation="处理加减",
                inputs=[current_expr],
                result="处理加减运算",
                formula="从左到右处理加减"
            ))
            
            # 计算加减表达式
            try:
                final_result = eval(current_expr)
                if isinstance(final_result, float) and final_result.is_integer():
                    final_result = int(final_result)
                
                # 如果有加减运算，记录步骤
                if '+' in current_expr or '-' in current_expr:
                    steps.append(MixedStep(
                        description="计算加减运算",
                        operation="加减运算",
                        inputs=[current_expr],
                        result=str(final_result),
                        formula=f"{current_expr} = {final_result}"
                    ))
                
                return final_result
            except Exception as e:
                raise ValueError(f"加减运算计算失败: {current_expr}")
        else:
            # 只剩一个数字
            return float(current_expr)



"""
混合运算算子模块
"""

from .parentheses_evaluator import ParenthesesEvaluator
from .zero_one_optimizer import ZeroOneOptimizer
from .mul_div_left_to_right import MulDivLeftToRightCalculator
from .add_sub_left_to_right import AddSubLeftToRightCalculator
from .standard_order_evaluator import StandardOrderEvaluator
from .bracketed_mix_evaluator import BracketedMixEvaluator
from .distribution_optimizer import DistributionOptimizer
from .complement_rewriting import ComplementRewritingCalculator

__all__ = [
    'ParenthesesEvaluator',
    'ZeroOneOptimizer', 
    'MulDivLeftToRightCalculator',
    'AddSubLeftToRightCalculator',
    'StandardOrderEvaluator',
    'BracketedMixEvaluator',
    'DistributionOptimizer',
    'ComplementRewritingCalculator'
]



"""
小数计算通用算子
实现小数的加减乘除基本运算规则
"""

from typing import List, Tuple
from decimal import Decimal, ROUND_HALF_UP
from ..base_types import MathCalculator, Formula, CalculationStep, ElementType


class DecimaCalculator(MathCalculator):
    """小数计算通用算子"""
    
    def __init__(self, name="小数计算", description="小数的加减乘除运算", priority=3):
        super().__init__(name, description, priority)
        self.decimal_precision = 4  # 默认精度
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：包含小数的四则运算"""
        if formula.type not in ["addition", "subtraction", "multiplication", "division"]:
            return False
        
        numbers = formula.get_numbers()
        if len(numbers) != 2:
            return False
        
        try:
            # 检查是否至少有一个小数
            has_decimal = False
            for elem in numbers:
                value = elem.get_numeric_value()
                if isinstance(value, float) and not value.is_integer():
                    has_decimal = True
                    break
                # 检查字符串表示是否包含小数点
                if hasattr(elem, 'text') and '.' in str(elem.text):
                    has_decimal = True
                    break
            
            return has_decimal
        except:
            return False
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建小数计算步骤"""
        if formula.type == "addition":
            return self._construct_addition_steps(formula)
        elif formula.type == "subtraction":
            return self._construct_subtraction_steps(formula)
        elif formula.type == "multiplication":
            return self._construct_multiplication_steps(formula)
        elif formula.type == "division":
            return self._construct_division_steps(formula)
        else:
            return []
    
    def _construct_addition_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建小数加法步骤"""
        numbers = formula.get_numbers()
        a, b = [elem.get_numeric_value() for elem in numbers]
        
        steps = []
        
        # 步骤1：对齐小数点
        a_str, b_str = self._align_decimals(a, b)
        steps.append(CalculationStep(
            description=f"对齐小数点：{a} + {b} → {a_str} + {b_str}",
            operation="对齐小数点",
            result=f"{a_str} + {b_str}"
        ))
        
        # 步骤2：按整数相加
        a_int = int(a_str.replace('.', ''))
        b_int = int(b_str.replace('.', ''))
        sum_int = a_int + b_int
        
        steps.append(CalculationStep(
            description=f"按整数相加：{a_int} + {b_int} = {sum_int}",
            operation="整数相加",
            result=sum_int
        ))
        
        # 步骤3：恢复小数点
        decimal_places = len(a_str.split('.')[1]) if '.' in a_str else 0
        final_result = sum_int / (10 ** decimal_places)
        
        steps.append(CalculationStep(
            description=f"恢复小数点：{sum_int} → {final_result}",
            operation="恢复小数点",
            result=final_result,
            formula="小数加法 = 对齐小数点 → 整数相加 → 恢复小数点"
        ))
        
        return steps
    
    def _construct_subtraction_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建小数减法步骤"""
        numbers = formula.get_numbers()
        a, b = [elem.get_numeric_value() for elem in numbers]
        
        steps = []
        
        # 步骤1：对齐小数点
        a_str, b_str = self._align_decimals(a, b)
        steps.append(CalculationStep(
            description=f"对齐小数点：{a} - {b} → {a_str} - {b_str}",
            operation="对齐小数点",
            result=f"{a_str} - {b_str}"
        ))
        
        # 步骤2：按整数相减
        a_int = int(a_str.replace('.', ''))
        b_int = int(b_str.replace('.', ''))
        diff_int = a_int - b_int
        
        steps.append(CalculationStep(
            description=f"按整数相减：{a_int} - {b_int} = {diff_int}",
            operation="整数相减",
            result=diff_int
        ))
        
        # 步骤3：恢复小数点
        decimal_places = len(a_str.split('.')[1]) if '.' in a_str else 0
        final_result = diff_int / (10 ** decimal_places)
        
        steps.append(CalculationStep(
            description=f"恢复小数点：{diff_int} → {final_result}",
            operation="恢复小数点",
            result=final_result,
            formula="小数减法 = 对齐小数点 → 整数相减 → 恢复小数点"
        ))
        
        return steps
    
    def _construct_multiplication_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建小数乘法步骤"""
        numbers = formula.get_numbers()
        a, b = [elem.get_numeric_value() for elem in numbers]
        
        steps = []
        
        # 步骤1：记录小数位数
        a_str, b_str = str(a), str(b)
        p1 = len(a_str.split('.')[1]) if '.' in a_str else 0
        p2 = len(b_str.split('.')[1]) if '.' in b_str else 0
        total_places = p1 + p2
        
        steps.append(CalculationStep(
            description=f"记录小数位数：{a}有{p1}位，{b}有{p2}位，共{total_places}位",
            operation="统计小数位数",
            result=f"总小数位数：{total_places}"
        ))
        
        # 步骤2：去小数点按整数相乘
        a_int = int(a_str.replace('.', ''))
        b_int = int(b_str.replace('.', ''))
        product_int = a_int * b_int
        
        steps.append(CalculationStep(
            description=f"整数相乘：{a_int} × {b_int} = {product_int}",
            operation="整数相乘",
            result=product_int
        ))
        
        # 步骤3：移动小数点
        final_result = product_int / (10 ** total_places)
        
        steps.append(CalculationStep(
            description=f"移动小数点{total_places}位：{product_int} → {final_result}",
            operation="移动小数点",
            result=final_result,
            formula="小数乘法位数 = 因数1小数位数 + 因数2小数位数"
        ))
        
        return steps
    
    def _construct_division_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建小数除法步骤"""
        numbers = formula.get_numbers()
        a, b = [elem.get_numeric_value() for elem in numbers]
        
        steps = []
        
        # 步骤1：统计除数小数位数
        b_str = str(b)
        decimal_places = len(b_str.split('.')[1]) if '.' in b_str else 0
        
        if decimal_places > 0:
            steps.append(CalculationStep(
                description=f"除数{b}有{decimal_places}位小数，需要化为整数",
                operation="分析除数",
                result=f"需要移动{decimal_places}位小数点"
            ))
            
            # 步骤2：同时移动小数点
            scale_factor = 10 ** decimal_places
            a_scaled = a * scale_factor
            b_scaled = b * scale_factor
            
            steps.append(CalculationStep(
                description=f"同时扩大{scale_factor}倍：{a} ÷ {b} → {a_scaled} ÷ {b_scaled}",
                operation="化除数为整数",
                result=f"{a_scaled} ÷ {b_scaled}"
            ))
        else:
            a_scaled, b_scaled = a, b
            steps.append(CalculationStep(
                description=f"除数{b}已是整数，直接计算",
                operation="除数分析",
                result=f"{a} ÷ {b}"
            ))
        
        # 步骤3：执行除法
        final_result = a_scaled / b_scaled
        
        steps.append(CalculationStep(
            description=f"执行除法：{a_scaled} ÷ {b_scaled} = {final_result}",
            operation="执行除法",
            result=final_result,
            formula="小数除法 = 除数化整数 → 同时移动小数点 → 执行除法"
        ))
        
        return steps
    
    def _align_decimals(self, a: float, b: float) -> Tuple[str, str]:
        """对齐小数点，短的补零"""
        a_str, b_str = str(a), str(b)
        
        # 获取小数位数
        a_places = len(a_str.split('.')[1]) if '.' in a_str else 0
        b_places = len(b_str.split('.')[1]) if '.' in b_str else 0
        
        # 统一小数位数
        max_places = max(a_places, b_places)
        
        if '.' not in a_str:
            a_str += '.' + '0' * max_places
        else:
            a_str += '0' * (max_places - a_places)
            
        if '.' not in b_str:
            b_str += '.' + '0' * max_places
        else:
            b_str += '0' * (max_places - b_places)
        
        return a_str, b_str
    
    def _format_decimal_result(self, result: float) -> float:
        """格式化小数结果，去除不必要的尾随零"""
        # 使用 Decimal 进行精确计算
        decimal_result = Decimal(str(result)).quantize(
            Decimal('0.0001'), rounding=ROUND_HALF_UP
        )
        return float(decimal_result)



"""
分数算子注册器
"""

from typing import List, Dict, Any, Optional
from .base_types import FractionCalculator, FractionProblem


class FractionCalculatorRegistry:
    from .calculators.ratio_to_value import RatioToValueCalculator
    # 移除类体 import，改为方法体 import
    """分数算子注册器"""
    
    def __init__(self):
        self.calculators: List[FractionCalculator] = []
        self._register_default_calculators()
    
    def _register_default_calculators(self):
        """注册默认的分数算子"""
        from .calculators.ratio_equation import RatioEquationCalculator
        from .calculators.integer_fraction_compare import IntegerFractionCompareCalculator
        from .calculators.ratio_to_fraction import RatioToFractionCalculator
        from .calculators.reduce_fraction import ReduceFractionCalculator
        from .calculators.reciprocal import ReciprocalCalculator
        from .calculators.reciprocal_equation import ReciprocalEquationCalculator
        from .calculators.decimal_to_fraction import DecimalToFractionCalculator
        from .calculators.same_denominator_add import SameDenominatorAddCalculator
        from .calculators.same_denominator_subtract import SameDenominatorSubtractCalculator
        from .calculators.different_denominator_add import DifferentDenominatorAddCalculator
        from .calculators.different_denominator_subtract import DifferentDenominatorSubtractCalculator
        from .calculators.fraction_multiply import FractionMultiplyCalculator
        from .calculators.fraction_integer_multiply import FractionIntegerMultiplyCalculator
        from .calculators.fraction_divide import FractionDivideCalculator
        from .calculators.fraction_compare import FractionCompareCalculator
        from .calculators.same_numerator_compare import SameNumeratorCompareCalculator
        from .calculators.cross_multiply_compare import CrossMultiplyCompareCalculator
        from .calculators.expression_compare import ExpressionCompareCalculator
        from .calculators.fraction_convert import FractionConvertCalculator
        from .calculators.mixed_improper_convert import MixedImproperConvertCalculator
        from .calculators.fraction_to_decimal import FractionToDecimalCalculator
        from .calculators.ratio_to_value import RatioToValueCalculator
        
        # 按优先级注册算子（优先级高的先注册）
        calculators = [
            RatioToValueCalculator(),
            RatioEquationCalculator(),
            FractionToDecimalCalculator(),
            # 新增算子
            IntegerFractionCompareCalculator(),
            RatioToFractionCalculator(),
            # 特殊算子
            ReciprocalEquationCalculator(),    # 9
            ReciprocalCalculator(),            # 8
            DecimalToFractionCalculator(),     # 11
            # 比较算子
            SameNumeratorCompareCalculator(),  # 2 - 同分子比较优先级最高
            FractionCompareCalculator(),       # 3 - 通用比较（包含同分母）
            CrossMultiplyCompareCalculator(),  # 4 - 交叉相乘
            ExpressionCompareCalculator(),     # 5 - 运算后比较
            # 运算算子
            SameDenominatorAddCalculator(),    # 6
            SameDenominatorSubtractCalculator(), # 6
            DifferentDenominatorAddCalculator(), # 7
            DifferentDenominatorSubtractCalculator(), # 7
            FractionMultiplyCalculator(),      # 5
            FractionIntegerMultiplyCalculator(), # 5
            FractionDivideCalculator(),        # 5
            # 转换算子
            ReduceFractionCalculator(),        # 10
            FractionConvertCalculator(),       # 12
            MixedImproperConvertCalculator(),  # 4
        ]
        
        for calc in calculators:
            self.register(calc)
    
    def register(self, calculator: FractionCalculator):
        """注册算子"""
        self.calculators.append(calculator)
        # 按优先级排序（高优先级在前）
        self.calculators.sort(key=lambda x: x.priority, reverse=True)
    
    def find_matching_calculators(self, problem: FractionProblem) -> List[FractionCalculator]:
        """找到匹配的算子"""
        matching = []
        
        for calculator in self.calculators:
            match_result = calculator.is_match_pattern(problem)
            if match_result.get("matched", False):
                matching.append(calculator)
        
        return matching
    
    def get_best_calculator(self, problem: FractionProblem) -> Optional[FractionCalculator]:
        """获取最佳算子"""
        matching = self.find_matching_calculators(problem)
        
        if not matching:
            return None
        
        # 按优先级和匹配分数排序
        matching.sort(key=lambda x: (x.priority, x.is_match_pattern(problem).get("score", 0)), reverse=True)
        
        return matching[0]
    
    def get_all_calculators(self) -> List[FractionCalculator]:
        """获取所有注册的算子"""
        return self.calculators.copy()
    
    def get_calculator_by_name(self, name: str) -> Optional[FractionCalculator]:
        """根据名称获取算子"""
        for calc in self.calculators:
            if calc.name == name:
                return calc
        return None



"""
长度单位换算算子
"""

from typing import List, Dict
from ..base_types import UnitCalculator, UnitProblem, UnitType, UnitValue, ConversionStep


class LengthUnitCalculator(UnitCalculator):
    """长度单位换算算子"""
    
    def __init__(self):
        super().__init__("长度单位换算", "处理千米、米、分米、厘米、毫米之间的换算", UnitType.LENGTH)
    
    def _initialize_conversion_rules(self) -> Dict[str, Dict[str, float]]:
        """初始化长度单位换算规则"""
        return {
            # 千米换算
            "km": {"km": 1, "m": 1000, "dm": 10000, "cm": 100000, "mm": 1000000},
            "千米": {"千米": 1, "米": 1000, "分米": 10000, "厘米": 100000, "毫米": 1000000},
            "公里": {"公里": 1, "米": 1000, "分米": 10000, "厘米": 100000, "毫米": 1000000},
            
            # 米换算
            "m": {"km": 0.001, "m": 1, "dm": 10, "cm": 100, "mm": 1000},
            "米": {"千米": 0.001, "米": 1, "分米": 10, "厘米": 100, "毫米": 1000},
            
            # 分米换算
            "dm": {"km": 0.0001, "m": 0.1, "dm": 1, "cm": 10, "mm": 100},
            "分米": {"千米": 0.0001, "米": 0.1, "分米": 1, "厘米": 10, "毫米": 100},
            
            # 厘米换算
            "cm": {"km": 0.00001, "m": 0.01, "dm": 0.1, "cm": 1, "mm": 10},
            "厘米": {"千米": 0.00001, "米": 0.01, "分米": 0.1, "厘米": 1, "毫米": 10},
            
            # 毫米换算
            "mm": {"km": 0.000001, "m": 0.001, "dm": 0.01, "cm": 0.1, "mm": 1},
            "毫米": {"千米": 0.000001, "米": 0.001, "分米": 0.01, "厘米": 0.1, "毫米": 1},
        }
    
    def is_match_pattern(self, problem: UnitProblem) -> bool:
        """判断是否为长度单位换算问题"""
        return problem.unit_type == UnitType.LENGTH

    def solve(self, problem: UnitProblem) -> tuple[float, List[ConversionStep]]:
        """求解长度单位换算问题"""
        steps = []
        total_result = 0
        operator = "+"
        # 处理复合单位（如3m5dm）
        for source_value in problem.source_values:
            # 直接换算到目标单位
            conversion_factor = self.get_conversion_factor(source_value.unit, problem.target_unit)
            
            if conversion_factor is None:
                print(f"无法从{source_value.unit}换算到{problem.target_unit}")
                raise ValueError(f"无法从{source_value.unit}换算到{problem.target_unit}")
            
            converted_value = source_value.value * conversion_factor
            if operator == '-':
                total_result -= converted_value
            elif operator == '+':
                total_result += converted_value
            operator = source_value.operator
            
            # 生成标准化的换算公式
            formula = self._get_standard_formula(source_value.unit, problem.target_unit)
            
            # 生成换算步骤
            step = ConversionStep(
                description=f"换算{source_value.unit}到{problem.target_unit}",
                operation="单位换算",
                from_value=source_value,
                result=UnitValue(converted_value, problem.target_unit),
                formula=formula
            )
            steps.append(step)
        
        # 如果有多个源单位，添加求和步骤
        if len(problem.source_values) > 1:
            formula = ' '.join([str(v.value * self.get_conversion_factor(v.unit, problem.target_unit)) + v.operator for v in problem.source_values]).rstrip('+')
            step = ConversionStep(
                description=f"合并所有换算结果",
                operation="求和",
                from_value=UnitValue(0, problem.target_unit),
                result=UnitValue(total_result, problem.target_unit),
                formula=f"总和 = {formula}"
            )
            steps.append(step)
        
        return total_result, steps
    
    def _get_standard_formula(self, from_unit: str, to_unit: str) -> str:
        """获取标准化的换算公式（大单位 = 小单位倍数）"""
        # 定义单位大小顺序（从大到小）
        unit_hierarchy = {
            "km": 5, "千米": 5, "公里": 5,
            "m": 4, "米": 4,
            "dm": 3, "分米": 3,
            "cm": 2, "厘米": 2,
            "mm": 1, "毫米": 1
        }
        
        from_level = unit_hierarchy.get(from_unit, 0)
        to_level = unit_hierarchy.get(to_unit, 0)
        
        # 确定哪个是大单位，哪个是小单位
        if from_level > to_level:
            # 从大单位换到小单位
            big_unit, small_unit = from_unit, to_unit
            factor = self.get_conversion_factor(from_unit, to_unit)
        else:
            # 从小单位换到大单位
            big_unit, small_unit = to_unit, from_unit
            factor = 1 / self.get_conversion_factor(from_unit, to_unit) if self.get_conversion_factor(from_unit, to_unit) else 1
        
        # 生成标准公式
        if factor and factor != 1:
            if factor == int(factor):
                return f"1{big_unit} = {int(factor)}{small_unit}"
            else:
                return f"1{big_unit} = {factor}{small_unit}"
        else:
            return f"1{from_unit} = 1{to_unit}"



"""
近整十除法实现
处理接近整十数的除法
"""

from typing import List
from ..base_types import MathCalculator, Formula, CalculationStep


class NearRoundDivision(MathCalculator):
    """近整十除法算法"""
    
    def __init__(self):
        super().__init__("近整十除法", "近整十，先估后调", priority=5)
    
    def _find_nearest_round(self, num: int) -> int:
        """找到最接近的整十数"""
        return round(num / 10) * 10
    
    def _find_nearest_hundred(self, num: int) -> int:
        """找到最接近的整百数"""
        return round(num / 100) * 100
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：被除数接近整十数或整百数"""
        if formula.type != "division":
            return False
        
        numbers = formula.get_numbers()
        if len(numbers) != 2:
            return False
        
        # 如果被除数是5的倍数，可使用更高阶方法
        if numbers[0].get_numeric_value() % 5 == 0:
            return False
        
        try:
            dividend, divisor = [elem.get_numeric_value() for elem in numbers]
            if not (isinstance(dividend, int) and isinstance(divisor, int)):
                return False
            
            if divisor == 0:
                return False
            
            # 除数应该是单位数，便于心算
            if divisor > 12 or divisor < 2:
                return False
            
            # 检查被除数是否接近整十数
            nearest_ten = self._find_nearest_round(dividend)
            diff_ten = abs(dividend - nearest_ten)
            
            # 检查被除数是否接近整百数
            nearest_hundred = self._find_nearest_hundred(dividend)
            diff_hundred = abs(dividend - nearest_hundred)
            
            # 如果接近整十数（差值在5以内）或整百数（差值在20以内）
            is_near_ten = diff_ten <= 5 and nearest_ten != dividend and nearest_ten >= 10
            is_near_hundred = diff_hundred <= 20 and nearest_hundred != dividend and nearest_hundred >= 100
            
            return is_near_ten or is_near_hundred
        except:
            return False
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建近整十除法步骤"""
        numbers = formula.get_numbers()
        dividend, divisor = [elem.get_numeric_value() for elem in numbers]
        
        # 确保是整数
        dividend_int = int(dividend)
        divisor_int = int(divisor)
        
        steps = []
        
        steps.append(CalculationStep(
            description=f"识别近整十除法：{dividend_int} ÷ {divisor_int}",
            operation="识别模式",
            result="近整十，先估后调"
        ))
        
        # 选择最合适的基准数（整十或整百）
        nearest_ten = self._find_nearest_round(dividend_int)
        diff_ten = abs(dividend_int - nearest_ten)
        
        nearest_hundred = self._find_nearest_hundred(dividend_int)
        diff_hundred = abs(dividend_int - nearest_hundred)
        
        # 选择差值较小的基准数
        if diff_ten <= diff_hundred and nearest_ten >= 10:
            base_num = nearest_ten
            diff = dividend_int - base_num
            base_type = "整十数"
        else:
            base_num = nearest_hundred
            diff = dividend_int - base_num
            base_type = "整百数"
        
        steps.append(CalculationStep(
            description=f"选择最接近的{base_type}：{base_num}，差值为 {diff}",
            operation="选择基准",
            result=f"{dividend_int} = {base_num} + ({diff})"
        ))
        
        # 计算基准数的除法
        base_quotient = base_num / divisor_int
        
        steps.append(CalculationStep(
            description=f"先算基准数除法：{base_num} ÷ {divisor_int} = {base_quotient}",
            operation="基准计算",
            result=base_quotient
        ))
        
        # 计算差值的影响
        if diff != 0:
            diff_quotient = diff / divisor_int
            
            steps.append(CalculationStep(
                description=f"计算差值影响：{diff} ÷ {divisor_int} = {diff_quotient}",
                operation="差值计算",
                result=diff_quotient
            ))
            
            # 合并结果
            final_result = base_quotient + diff_quotient
            
            if diff > 0:
                steps.append(CalculationStep(
                    description=f"被除数比基准数大，加上差值：{base_quotient} + {diff_quotient} = {final_result}",
                    operation="调整结果",
                    result=final_result,
                    formula="近整数除法：基准数结果 + 差值结果"
                ))
            else:
                steps.append(CalculationStep(
                    description=f"被除数比基准数小，减去差值：{base_quotient} + {diff_quotient} = {final_result}",
                    operation="调整结果",
                    result=final_result,
                    formula="近整数除法：基准数结果 + 差值结果"
                ))
        else:
            final_result = base_quotient
            steps.append(CalculationStep(
                description=f"无需调整，结果就是：{final_result}",
                operation="确定结果",
                result=final_result
            ))
        
        # 提供验证
        actual_result = dividend_int / divisor_int
        if abs(final_result - actual_result) < 0.001:
            steps.append(CalculationStep(
                description=f"验证：{dividend_int} ÷ {divisor_int} = {final_result}",
                operation="验证正确",
                result="✓ 计算正确"
            ))
        else:
            steps.append(CalculationStep(
                description=f"精确结果：{dividend_int} ÷ {divisor_int} = {actual_result}",
                operation="精确计算",
                result=actual_result
            ))
        
        return steps


"""
智能计算器引擎
负责算法自动匹配和执行
"""

from typing import List, Dict, Any, Optional, Tuple
from .base_types import Formula, MathCalculator, CalculationStep, ElementType
from .formula_parser import FormulaParser
from .register import SmartCalculatorReister
from ..base.engine import BaseEngine

operators = ['+', '-', '*', '/', '×', '÷']

class SmartCalculatorEngine(SmartCalculatorReister, BaseEngine):
    """智能计算器引擎 - 算法自动匹配机制"""
    
    def __init__(self):
        super().__init__()

    def is_match_pattern(self, problem: str) -> bool:
        """判断是否为匹配的算术计算问题"""
        ## 是否包含加减乘除运算符号
        if not any(op in problem for op in operators):
            return False
        formula = FormulaParser.parse(problem)
        return any(calculator.is_match_pattern(formula) for calculator in self.calculators)
    
    def fix_number_representation(self, expression) -> str:
        """修正数字表示法"""
        # 如果是整数，直接返回字符串形式
        if isinstance(expression, int):
            return str(expression)
        if isinstance(expression, float):
            # 保留4位精度再变成字符串
            expression = f"{expression:.4f}"
        else:
            expression = str(expression)
        # 如果是0结尾，去掉末尾的所有0
        if '.' in expression:
            expression = expression.rstrip('0').rstrip('.')
        return expression
    def select_all_calculators(self, formula: Formula) -> Dict[str, Any]:
        """获取所有匹配算子的计算结果并进行交叉验证"""
        matching = self.find_matching_calculators(formula)
        
        if not matching:
            return {
                'matched_count': 0,
                'calculators': [],
                'cross_validation': {'passed': False, 'message': '没有匹配的算子'},
                'consensus_result': None
            }
        
        # 按优先级排序
        matching.sort(key=lambda x: x.priority, reverse=True)
        
        calculator_results = []
        results_for_validation = []
        
        for calc in matching:
            try:
                # 执行计算
                result, steps = calc.execute(formula)
                is_valid = calc.validate(formula)
                if not is_valid or not steps or len(steps) == 0:
                    continue
                calculator_info = {
                    'name': calc.name,
                    'description': calc.description,
                    'priority': calc.priority,
                    'result': self.fix_number_representation(result),
                    'steps': steps,
                    'step_count': len(steps),
                    'validation': is_valid,
                    'formula': getattr(calc, 'formula', None),
                    'success': True,
                    'error': None
                }
                
                calculator_results.append(calculator_info)
                results_for_validation.append(result)
                
                print(f"✓ {calc.name}: 结果={result}, 步骤数={len(steps)}")
                
            except Exception as e:
                error_info = {
                    'name': calc.name,
                    'description': calc.description,
                    'priority': calc.priority,
                    'result': None,
                    'steps': [],
                    'step_count': 0,
                    'validation': False,
                    'formula': None,
                    'success': False,
                    'error': str(e)
                }
                
                calculator_results.append(error_info)
                print(f"✗ {calc.name}: 计算失败 - {str(e)}")
        
        # 交叉验证：检查所有成功的计算结果是否一致
        successful_results = [r for r in results_for_validation if r is not None]
        cross_validation = self._perform_cross_validation(successful_results, formula)
        
        # 确定共识结果
        consensus_result = None
        if cross_validation['passed'] and successful_results:
            consensus_result = successful_results[0]  # 所有结果一致时取任意一个
        elif successful_results:
            # 结果不一致时，选择出现次数最多的结果
            from collections import Counter
            result_counts = Counter(successful_results)
            consensus_result = result_counts.most_common(1)[0][0]
        
        return {
            'matched_count': len(matching),
            'successful_count': len([r for r in calculator_results if r['success']]),
            'calculators': calculator_results,
            'cross_validation': cross_validation,
            'consensus_result': consensus_result,
            'formula_type': formula.type,
            'expression': str(formula)
        }
    
    def _perform_cross_validation(self, results: List, formula: Formula) -> Dict[str, Any]:
        """执行交叉验证"""
        if not results:
            return {
                'passed': False,
                'message': '没有可验证的结果',
                'unique_results': [],
                'result_distribution': {}
            }
        
        if len(results) == 1:
            # 只有一个结果时，与直接计算对比
            try:
                direct_result = formula.evaluate_direct()
                is_consistent = abs(results[0] - direct_result) < 1e-10  # 浮点数比较
                return {
                    'passed': is_consistent,
                    'message': '与直接计算结果一致' if is_consistent else '与直接计算结果不一致',
                    'unique_results': [results[0]] if is_consistent else [results[0], direct_result],
                    'result_distribution': {str(results[0]): 1},
                    'direct_calculation': direct_result
                }
            except:
                return {
                    'passed': True,  # 无法直接计算时认为通过
                    'message': '无法进行直接计算验证',
                    'unique_results': [results[0]],
                    'result_distribution': {str(results[0]): 1}
                }
        
        # 多个结果的情况
        from collections import Counter
        
        # 统计结果分布
        result_counts = Counter(results)
        unique_results = list(result_counts.keys())
        
        # 检查是否所有结果都一致
        all_consistent = len(unique_results) == 1
        
        # 计算一致性比例
        total_results = len(results)
        max_count = max(result_counts.values())
        consistency_ratio = max_count / total_results
        
        if all_consistent:
            message = f'所有 {total_results} 个算子结果完全一致'
            passed = True
        elif consistency_ratio >= 0.8:  # 80%以上一致认为通过
            message = f'{max_count}/{total_results} 个算子结果一致 (一致性: {consistency_ratio:.1%})'
            passed = True
        else:
            message = f'结果不一致，最多 {max_count}/{total_results} 个算子一致 (一致性: {consistency_ratio:.1%})'
            passed = False
        
        # 与直接计算结果对比
        try:
            direct_result = formula.evaluate_direct()
            most_common_result = result_counts.most_common(1)[0][0]
            direct_match = abs(most_common_result - direct_result) < 1e-10
            
            if direct_match:
                message += '，且与直接计算一致'
            else:
                message += '，但与直接计算不一致'
                passed = False
        except:
            message += '，无法验证直接计算'
        
        return {
            'passed': passed,
            'message': message,
            'unique_results': unique_results,
            'result_distribution': {str(k): v for k, v in result_counts.items()},
            'consistency_ratio': consistency_ratio,
            'total_calculators': total_results
        }

    def select_best_calculator(self, formula: Formula) -> Optional[MathCalculator]:
        """根据优先级和步骤数选择最佳算子"""
        matching = self.find_matching_calculators(formula)
        
        if not matching:
            return None
        
        # 按优先级排序（高优先级在前）
        matching.sort(key=lambda x: x.priority, reverse=True)
        # 获取最高优先级
        highest_priority = matching[0].priority
        
        # 筛选出最高优先级的算子
        top_priority_calcs = [calc for calc in matching if calc.priority == highest_priority]
        
        if len(top_priority_calcs) == 1:
            return top_priority_calcs[0]
        
        # 如果优先级相同，比较步骤数量
        best_calc = None
        min_steps = float('inf')
        
        for calc in top_priority_calcs:
            try:
                steps = calc.construct_steps(formula)
                print(f"算子 {calc.name} 生成步骤数: {len(steps)}")
                if len(steps) < min_steps:
                    min_steps = len(steps)
                    best_calc = calc
            except:
                continue
        return best_calc
    
    def solve(self, expression: str) -> Dict[str, Any]:
            return self.calculate(expression)
    def solve_all_solutions(self, expression: str) -> Dict[str, Any]:
            return self.select_all_calculators(FormulaParser.parse(expression))
    def format_formula(self, expression) -> Formula:
        return FormulaParser.parse(expression)
    def calculate(self, expression: str) -> Dict[str, Any]:
        """主计算方法"""
        try:
            # 检查空表达式
            if not expression or not expression.strip():
                return {
                    'success': False,
                    'expression': expression,
                    'error': '表达式不能为空',
                    'method': None,
                    'result': None
                }
            
            # 解析表达式
            formula = self.format_formula(expression)
            # 检查表达式是否有效
            if not formula.is_valid_expression():
                return {
                    'success': False,
                    'expression': expression,
                    'error': '无效的表达式',
                    'method': None,
                    'result': None
                }
            
            # 找到最佳算子
            best_calculator = self.select_best_calculator(formula)
            
            if best_calculator is None:
                # 没找到匹配的算子，使用直接计算
                try:
                    direct_result = formula.evaluate_direct()
                    return {
                        'success': True,
                        'expression': expression,
                        'method': '直接计算',
                        'description': '使用标准算术运算',
                        'result': self.format_result(direct_result),
                        'steps': [],
                        'formula_type': formula.type,
                        'validation': True,
                        'priority': 0
                    }
                except:
                    return {
                        'success': False,
                        'expression': expression,
                        'error': '无法计算表达式',
                        'method': None,
                        'result': None
                    }
            
            # 执行计算
            result, steps = best_calculator.execute(formula)
            
            # 验证结果
            is_valid = best_calculator.validate(formula)
            
            return {
                'success': True,
                'expression': expression,
                'method': best_calculator.name,
                'description': best_calculator.description,
                'result': self.format_result(result),
                'steps': steps,
                'name': best_calculator.name,
                'formula': best_calculator.formula,
                'formula_type': formula.type,
                'validation': is_valid,
                'priority': best_calculator.priority
            }
            
        except Exception as e:
            return {
                'success': False,
                'expression': expression,
                'error': str(e),
                'method': None,
                'result': None
            }

    def batch_calculate(self, expressions: List[str]) -> List[Dict[str, Any]]:
        """批量计算"""
        results = []
        for expr in expressions:
            result = self.calculate(expr)
            results.append(result)
        return results
    
    def get_available_methods(self) -> List[Dict[str, Any]]:
        """获取所有可用的速算方法"""
        methods = []
        for calc in self.calculators:
            methods.append({
                'name': calc.name,
                'description': calc.description,
                'priority': calc.priority
            })
        return sorted(methods, key=lambda x: x['priority'], reverse=True)
    
    def calculate_with_cross_validation(self, expression: str) -> Dict[str, Any]:
        """带交叉验证的计算方法"""
        try:
            # 检查空表达式
            if not expression or not expression.strip():
                return {
                    'success': False,
                    'expression': expression,
                    'error': '表达式不能为空',
                    'cross_validation': None
                }
            
            # 解析表达式
            formula = FormulaParser.parse(expression)
            
            # 检查表达式是否有效
            if not formula.is_valid_expression():
                return {
                    'success': False,
                    'expression': expression,
                    'error': '无效的表达式',
                    'cross_validation': None
                }
            
            # 获取所有算子的计算结果
            all_results = self.select_all_calculators(formula)
            
            # 选择最佳算子
            best_calculator = self.select_best_calculator(formula)
            
            # 构建返回结果
            result_data = {
                'success': True,
                'expression': expression,
                'formula_type': formula.type,
                'all_results': all_results,
                'best_method': None,
                'final_result': all_results['consensus_result']
            }
            
            # 添加最佳算子信息
            if best_calculator:
                try:
                    best_result, best_steps = best_calculator.execute(formula)
                    result_data['best_method'] = {
                        'name': best_calculator.name,
                        'description': best_calculator.description,
                        'result': best_result,
                        'steps': best_steps,
                        'priority': best_calculator.priority,
                        'validation': best_calculator.validate(formula)
                    }
                except Exception as e:
                    result_data['best_method'] = {
                        'error': f'最佳算子执行失败: {str(e)}'
                    }
            
            return result_data
            
        except Exception as e:
            return {
                'success': False,
                'expression': expression,
                'error': str(e),
                'cross_validation': None
            }
        
class DivisionRemainderEngine(SmartCalculatorEngine):
    splitor = '······'
    """带余数除法计算引擎"""
    def __init__(self):
        super().__init__()
        self.calculators = [self.get_division_remainder()]
    def format_formula(self, expression) -> Formula:
        formula = FormulaParser.parse(expression)
        if '@' not in formula.original_expression:
            formula.original_expression = expression.replace(' ', '')
        return formula
    def is_match_pattern(self, problem: str) -> bool:
        operators = '><='
        if  ('@×'  in problem or '×@' in problem) and any(op in problem for op in operators):
            return True
        if '······' not in problem:
            return False
        return True
    
class RevertCalculatorEngine(SmartCalculatorEngine):
    """未知数逆运算计算引擎"""
    def __init__(self):
        super().__init__()
        self.calculators = [self.get_revert_calculator()]
    def format_formula(self, expression) -> Formula:
        formula = FormulaParser.parse(expression)
        formula.original_expression = expression.replace(' ', '')
        return formula
    def is_match_pattern(self, problem: str) -> bool:
        if problem and (problem[0] == '@' or '@=' in problem):
            return True
        formula = self.format_formula(problem)
        return any([calculator.is_match_pattern(formula) for calculator in self.calculators])


"""
分数表达式解析器
"""

import re
from typing import List, Optional
from .base_types import FractionProblem, FractionOperand, OperationType, FractionUtils


class FractionParser:
    """分数表达式解析器"""
    
    @staticmethod
    def parse(expression: str) -> FractionProblem:
        """解析分数表达式"""
        expression = expression.strip()

        ## 表达式兼容处理
        expression = expression.replace(r'$\frac{', r'#frac{').replace(r'}$', r'}#')
        if re.search(r'#frac\{[\d\-]+\}\{[\d\-]+\}(?=[^#])', expression):
            expression = re.sub(r'#frac\{([\d\-]+)\}\{([\d\-]+)\}(?=[^#])', r'#frac{\1}{\2}#', expression)
        
        # 检查表达式是否包含分数格式
        if not FractionParser._has_fraction_format(expression) and ':' not in expression:
            raise ValueError(f"表达式不包含分数格式: {expression}")
        
        # 识别运算类型和操作数 - 调整@符号的处理优先级
        if ':' in expression:
            # 查看是否有两个：
            if expression.count(':') == 2 and '@' in expression:
                problem = FractionParser._parse_ratio_equation(expression)
                problem.id = "ratio_equation"
                return problem
            # a:b=@ 结构，直接转比值转数值问题
            if '=' in expression and '=@' in expression:
                left, right = expression.split('=')
                left_parts = [p.strip() for p in left.split(':')]
                if len(left_parts) == 2:
                    # 构造结构化问题
                    from fractions import Fraction
                    from .base_types import FractionOperand, FractionType
                    op1 = FractionUtils.parse_fraction_text(left_parts[0])
                    op2 = FractionUtils.parse_fraction_text(left_parts[1])
                    op3 = FractionOperand(
                        raw='@',
                        fraction=Fraction(0, 1),
                        fraction_type=FractionType.INTEGER,
                        original_format="unknown"
                    )
                    op3.__dict__['is_unknown'] = True   
                    operands = [op1, op2, op3]
                    return FractionProblem(
                        id="ratio_to_value",
                        original_text=expression,
                        operation=OperationType.CONVERT,
                        operands=operands
                    )
            return FractionParser._parse_convert(expression.split('=')[0])
        if '=' in expression and '@' in expression:
            return FractionParser._parse_equation(expression)
        elif '@' in expression:  # 添加@符号比较的处理
            return FractionParser._parse_comparison(expression)
        elif any(op in expression for op in ['>', '<', '≥', '≤']):
            return FractionParser._parse_comparison(expression)
        elif '+' in expression:
            return FractionParser._parse_addition(expression)
        elif '-' in expression:
            return FractionParser._parse_subtraction(expression)
        elif '×' in expression or '*' in expression:
            return FractionParser._parse_multiplication(expression)
        elif '÷' in expression or '/' in expression:
            return FractionParser._parse_division(expression)
        else:
            return FractionParser._parse_single_operand(expression)
    
    @staticmethod
    def _has_fraction_format(expression: str) -> bool:
        """检查表达式是否包含分数格式"""
        # 检查LaTeX分数格式
        if re.search(r'#[\d\-]*frac\{[\d@]+\}', expression):
            return True
        # 检查普通分数格式（但要确保不是纯整数）
        if re.search(r'\d+/\d+', expression):
            return True
        # 检查带分数格式
        if re.search(r'#\d+frac\{\d+\}\{\d+\}#', expression):
            return True
        # 如果只包含整数和运算符，不认为是分数表达式
        if re.match(r'^[\d\s+\-×*/÷()]+$', expression):
            return False
        return False
    
    @staticmethod
    def _parse_equation(expression: str) -> FractionProblem:
        """解析等式（包含@的题目）"""
        left, right = expression.split('=')
        left_parts = FractionParser._extract_operands(left)
        
        # 确定运算类型
        if '+' in left:
            operation = OperationType.ADD
        elif '-' in left:
            operation = OperationType.SUBTRACT
        elif '×' in left or '*' in left:
            operation = OperationType.MULTIPLY
        elif '÷' in left or '/' in left:
            operation = OperationType.DIVIDE
        else:
            operation = OperationType.CONVERT
        
        # 解析操作数
        operands = []
        for part in left_parts:
            if part != '@' and not part in ['+', '-', '×', '*', '÷', '/', ':']:
                try:
                    operands.append(FractionUtils.parse_fraction_text(part))
                except ValueError:
                    # 跳过无法解析的部分
                    continue
        
        # 确定目标格式
        target_format = "fraction"
        if '#frac{@}{@}#' in right:
            target_format = "latex_fraction"
        elif '@%' in right:
            target_format = "percent"
        elif '@' in right and '.' in right:
            target_format = "decimal"
        
        return FractionProblem(
            id="equation",
            original_text=expression,
            operation=operation,
            operands=operands,
            target_format=target_format
        )
    
    @staticmethod
    def _parse_comparison(expression: str) -> FractionProblem:
        """解析比较表达式"""
        # 先处理@符号的比较
        if '@' in expression:
            left, right = expression.split('@')
            operands = [
                FractionUtils.parse_fraction_text(left.strip()),
                FractionUtils.parse_fraction_text(right.strip())
            ]
            return FractionProblem(
                id="comparison",
                original_text=expression,
                operation=OperationType.COMPARE,
                operands=operands
            )
        
        # 然后处理传统的比较符号
        for op in ['≥', '≤', '>', '<']:
            if op in expression:
                left, right = expression.split(op)
                operands = [
                    FractionUtils.parse_fraction_text(left.strip()),
                    FractionUtils.parse_fraction_text(right.strip())
                ]
                return FractionProblem(
                    id="comparison",
                    original_text=expression,
                    operation=OperationType.COMPARE,
                    operands=operands
                )
        
        raise ValueError(f"无法解析比较表达式: {expression}")
    
    @staticmethod
    def _parse_addition(expression: str) -> FractionProblem:
        """解析加法表达式"""
        parts = expression.split('+')
        operands = []
        for part in parts:
            try:
                operands.append(FractionUtils.parse_fraction_text(part.strip()))
            except ValueError:
                # 跳过无法解析的部分
                continue
        
        if len(operands) < 2:
            raise ValueError(f"加法表达式需要至少两个操作数: {expression}")
        
        return FractionProblem(
            id="addition",
            original_text=expression,
            operation=OperationType.ADD,
            operands=operands
        )
    
    @staticmethod
    def _parse_subtraction(expression: str) -> FractionProblem:
        """解析减法表达式"""
        parts = expression.split('-')
        operands = []
        for part in parts:
            if part.strip():
                try:
                    operands.append(FractionUtils.parse_fraction_text(part.strip()))
                except ValueError:
                    continue
        
        if len(operands) < 2:
            raise ValueError(f"减法表达式需要至少两个操作数: {expression}")
        
        return FractionProblem(
            id="subtraction",
            original_text=expression,
            operation=OperationType.SUBTRACT,
            operands=operands
        )
    
    @staticmethod
    def _parse_multiplication(expression: str) -> FractionProblem:
        """解析乘法表达式"""
        for op in ['×', '*']:
            if op in expression:
                parts = expression.split(op)
                operands = []
                for part in parts:
                    try:
                        operands.append(FractionUtils.parse_fraction_text(part.strip()))
                    except ValueError:
                        continue
                
                if len(operands) < 2:
                    raise ValueError(f"乘法表达式需要至少两个操作数: {expression}")
                
                return FractionProblem(
                    id="multiplication", 
                    original_text=expression,
                    operation=OperationType.MULTIPLY,
                    operands=operands
                )
        
        raise ValueError(f"无法解析乘法表达式: {expression}")
    
    @staticmethod
    def _parse_division(expression: str) -> FractionProblem:
        """解析除法表达式"""
        for op in ['÷', '/']:
            if op in expression:
                parts = expression.split(op)
                operands = []
                for part in parts:
                    try:
                        operands.append(FractionUtils.parse_fraction_text(part.strip()))
                    except ValueError:
                        continue
                
                if len(operands) < 2:
                    raise ValueError(f"除法表达式需要至少两个操作数: {expression}")
                
                return FractionProblem(
                    id="division",
                    original_text=expression,
                    operation=OperationType.DIVIDE,
                    operands=operands
                )
        
        raise ValueError(f"无法解析除法表达式: {expression}")
    @staticmethod
    def _parse_ratio_equation(expression: str) -> FractionProblem:
        """解析比值等值方程 a:b = c:d，支持@未知数"""
        left, right = expression.split('=')
        left_parts = [p.strip() for p in left.split(':')]
        right_parts = [p.strip() for p in right.split(':')]
        if len(left_parts) != 2 or len(right_parts) != 2:
            raise ValueError(f"比值等值表达式格式错误: {expression}")
        operands = []
        for part in left_parts + right_parts:
            if part == '@':
                # 构造未知数操作数
                from fractions import Fraction
                from .base_types import FractionOperand, FractionType
                op = FractionOperand(
                    raw='@',
                    fraction=Fraction(0, 1),
                    fraction_type=FractionType.INTEGER,
                    original_format="unknown"
                )
                op.__dict__['is_unknown'] = True
                operands.append(op)
            else:
                op = FractionUtils.parse_fraction_text(part)
                operands.append(op)
        if len(operands) != 4:
            raise ValueError(f"比值等值表达式需要4个操作数: {expression}")
        return FractionProblem(
            id="ratio_equation",
            original_text=expression,
            operation=OperationType.CONVERT,
            operands=operands
        )
    @staticmethod
    def _parse_convert(expression: str) -> FractionProblem:
        """解析除法表达式"""
        for op in [':']:
            if op in expression:
                parts = expression.split(op)
                operands = []
                for part in parts:
                    try:
                        operands.append(FractionUtils.parse_fraction_text(part.strip()))
                    except ValueError:
                        continue
                
                if len(operands) < 2:
                    raise ValueError(f"除法表达式需要至少两个操作数: {expression}")
                
                return FractionProblem(
                    id="division",
                    original_text=expression,
                    operation=OperationType.CONVERT,
                    operands=operands
                )
        
        raise ValueError(f"无法解析除法表达式: {expression}")
    
    @staticmethod
    def _parse_single_operand(expression: str) -> FractionProblem:
        """解析单操作数（如约分、转换等）"""
        operand = FractionUtils.parse_fraction_text(expression)
        
        return FractionProblem(
            id="single",
            original_text=expression,
            operation=OperationType.REDUCE,
            operands=[operand]
        )
    
    @staticmethod
    def _extract_operands(expression: str) -> List[str]:
        """提取表达式中的操作数和运算符"""
        # 使用正则表达式分割，保留分隔符
        pattern = r'(#\d*frac\{\d+\}\{\d+\}#|\d+/\d+|\d+\.\d+|\d+%?|[@+\-×*/÷])'
        parts = re.findall(pattern, expression)
        return [part for part in parts if part.strip()]



"""
混合运算算子系统
"""

from .engine import MixedEngine
from .base_types import MixedProblem, MixedResult, MixedCalculator, MixedStep, OperationType

__all__ = ['MixedEngine', 'MixedProblem', 'MixedResult', 'MixedCalculator', 'MixedStep', 'OperationType']



"""
连续数相加实现
处理两个连续整数的加法，如49+50=99
"""

from typing import List
from ..base_types import MathCalculator, Formula, CalculationStep


class ConsecutiveNumberAddition(MathCalculator):
    """连续数相加算法"""
    
    def __init__(self):
        super().__init__("连续数相加", "两个连续整数的相加", priority=8)
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：两个连续的整数相加"""
        if formula.type != "addition":
            return False
        
        numbers = formula.get_numbers()
        if len(numbers) != 2:
            return False
        
        try:
            addend1, addend2 = [elem.get_numeric_value() for elem in numbers]
            if not (isinstance(addend1, int) and isinstance(addend2, int)):
                return False
            
            # 检查是否为连续数（差值为1）
            return abs(addend1 - addend2) == 1
        except:
            return False
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建连续数相加步骤"""
        numbers = formula.get_numbers()
        addend1 = int(numbers[0].get_numeric_value())
        addend2 = int(numbers[1].get_numeric_value())
        
        # 确保第一个数是较小的
        if addend1 > addend2:
            addend1, addend2 = addend2, addend1
        
        steps = []
        
        steps.append(CalculationStep(
            description=f"{addend1} + {addend2} 识别连续数相加",
            operation="识别连续数",
            result=f"{addend1}和{addend2}是连续整数"
        ))
        
        steps.append(CalculationStep(
            description=f"连续整数相加 = 2n + 1",
            operation="应用连续数相加规律",
            result=f"n = {addend1}, 2n + 1 = {2 * addend1 + 1}"
        ))
        
        result = addend1 + addend2
        steps.append(CalculationStep(
            description=f"{addend1} + {addend2} = {result}",
            operation="计算结果",
            result=result,
            formula="n + (n+1) = 2n + 1"
        ))
        
        return steps


"""
反用加法实现
将减法转换为加法来计算，特别适用于被减数小于减数的情况
或者通过添加相同数值来简化计算
"""

from typing import List
from ..base_types import MathCalculator, Formula, CalculationStep


class InverseAdditionSubtraction(MathCalculator):
    """反用加法算法"""
    
    def __init__(self):
        super().__init__("反用加法", "将减法转换为加法计算", priority=5)
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：适合转换为加法的减法运算"""
        if formula.type != "subtraction":
            return False
        
        numbers = formula.get_numbers()
        if len(numbers) != 2:
            return False
        
        try:
            values = [elem.get_numeric_value() for elem in numbers]
            if not all(isinstance(v, (int, float)) for v in values):
                return False
            
            minuend, subtrahend = values
            
            # 情况1：被减数小于减数（结果为负）
            if minuend < subtrahend:
                return True
            
            # 情况2：通过加法验算更简单
            # 检查是否存在便于加法计算的数值关系
            diff = minuend - subtrahend
            if diff > 0:
                # 如果差值较小且容易通过加法验证
                if diff < min(minuend, subtrahend) / 2:
                    return True
                    
                # 如果减数接近某个整数倍关系
                for base in [10, 100, 1000]:
                    if abs(subtrahend % base) <= 3 or abs(base - (subtrahend % base)) <= 3:
                        return True
            
            return False
            
        except:
            return False
    
    def _find_addition_strategy(self, minuend: float, subtrahend: float) -> dict:
        """寻找最适合的反用加法策略"""
        strategies = []
        
        # 策略1：直接转换（当被减数小于减数时）
        if minuend < subtrahend:
            strategies.append({
                'type': 'negative_result',
                'description': f'被减数小于减数，结果为负数',
                'calculation': f'{subtrahend} - {minuend} = {subtrahend - minuend}',
                'final_result': minuend - subtrahend,
                'ease_score': 9
            })
        
        # 策略2：加法验算法
        result = minuend - subtrahend
        if result > 0:
            strategies.append({
                'type': 'addition_check',
                'description': f'用加法验算：{subtrahend} + ? = {minuend}',
                'calculation': f'{subtrahend} + {result} = {minuend}',
                'result': result,
                'ease_score': 7
            })
        
        # 策略3：补数加法
        # 寻找接近整数倍的补数
        for base in [10, 100, 1000]:
            if subtrahend < base:
                complement = base - subtrahend
                if 1 <= complement <= base // 10:  # 补数不能太大
                    adjusted_minuend = minuend + complement
                    strategies.append({
                        'type': 'complement_addition',
                        'description': f'补数法：将 {subtrahend} 补至 {base}',
                        'complement': complement,
                        'base': base,
                        'adjusted_minuend': adjusted_minuend,
                        'calculation': f'({minuend} + {complement}) - {base} = {adjusted_minuend} - {base} = {adjusted_minuend - base}',
                        'result': adjusted_minuend - base,
                        'ease_score': 8 if base == 10 else 6
                    })
        
        # 策略4：分解加法
        # 将减数分解为更容易的加法组合
        if subtrahend >= 10:
            # 尝试分解为整十数和个位数
            tens = (int(subtrahend) // 10) * 10
            ones = int(subtrahend) % 10
            
            if ones != 0 and tens != 0:
                temp_result = minuend - tens
                final_result = temp_result - ones
                strategies.append({
                    'type': 'decompose_subtraction',
                    'description': f'分解减数：{subtrahend} = {tens} + {ones}',
                    'tens': tens,
                    'ones': ones,
                    'temp_result': temp_result,
                    'calculation': f'{minuend} - {tens} - {ones} = {temp_result} - {ones} = {final_result}',
                    'result': final_result,
                    'ease_score': 6
                })
        
        # 如果没有特殊策略，使用基本加法验算
        if not strategies:
            result = minuend - subtrahend
            strategies.append({
                'type': 'basic_check',
                'description': '基本加法验算',
                'calculation': f'{subtrahend} + {result} = {minuend}',
                'result': result,
                'ease_score': 5
            })
        
        return max(strategies, key=lambda x: x['ease_score'])
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建反用加法步骤"""
        numbers = formula.get_numbers()
        minuend = float(numbers[0].get_numeric_value())
        subtrahend = float(numbers[1].get_numeric_value())
        
        steps = []
        
        steps.append(CalculationStep(
            description=f"{minuend} - {subtrahend} 使用反用加法",
            operation="识别减法转加法",
            result="寻找最适合的加法转换策略"
        ))
        
        strategy = self._find_addition_strategy(minuend, subtrahend)
        
        if strategy['type'] == 'negative_result':
            steps.append(CalculationStep(
                description=f"被减数 {minuend} < 减数 {subtrahend}，结果为负数",
                operation="识别负数结果",
                result="转换为：-（减数 - 被减数）"
            ))
            
            positive_diff = subtrahend - minuend
            steps.append(CalculationStep(
                description=f"计算：{subtrahend} - {minuend} = {positive_diff}",
                operation="计算正差值",
                result=positive_diff
            ))
            
            steps.append(CalculationStep(
                description=f"最终结果：-{positive_diff} = {strategy['final_result']}",
                operation="添加负号",
                result=strategy['final_result'],
                formula="a - b = -(b - a) 当 a < b 时"
            ))
            
        elif strategy['type'] == 'addition_check':
            steps.append(CalculationStep(
                description=f"用加法验算：{subtrahend} + ? = {minuend}",
                operation="加法验算设置",
                result=f"寻找使等式成立的数值"
            ))
            
            result = strategy['result']
            steps.append(CalculationStep(
                description=f"验算：{subtrahend} + {result} = {minuend}",
                operation="加法验算",
                result=f"所以 {minuend} - {subtrahend} = {result}"
            ))
            
            steps.append(CalculationStep(
                description=f"最终结果：{result}",
                operation="确认结果",
                result=result,
                formula="减法验算：a - b = c ⟺ b + c = a"
            ))
            
        elif strategy['type'] == 'complement_addition':
            complement = strategy['complement']
            base = strategy['base']
            adjusted_minuend = strategy['adjusted_minuend']
            
            steps.append(CalculationStep(
                description=f"补数法：将 {subtrahend} 补至 {base}，需要加 {complement}",
                operation="确定补数",
                result=f"同时给被减数和减数都加 {complement}"
            ))
            
            steps.append(CalculationStep(
                description=f"调整后：({minuend} + {complement}) - ({subtrahend} + {complement}) = {adjusted_minuend} - {base}",
                operation="补数调整",
                result=f"{adjusted_minuend} - {base}"
            ))
            
            final_result = adjusted_minuend - base
            steps.append(CalculationStep(
                description=f"{adjusted_minuend} - {base} = {final_result}",
                operation="计算结果",
                result=final_result,
                formula=f"补数法：(a+k) - (b+k) = a - b"
            ))
            
        elif strategy['type'] == 'decompose_subtraction':
            tens = strategy['tens']
            ones = strategy['ones']
            temp_result = strategy['temp_result']
            
            steps.append(CalculationStep(
                description=f"分解减数：{subtrahend} = {tens} + {ones}",
                operation="减数分解",
                result=f"分步计算：先减 {tens}，再减 {ones}"
            ))
            
            steps.append(CalculationStep(
                description=f"第一步：{minuend} - {tens} = {temp_result}",
                operation="减去整十数",
                result=temp_result
            ))
            
            final_result = temp_result - ones
            steps.append(CalculationStep(
                description=f"第二步：{temp_result} - {ones} = {final_result}",
                operation="减去个位数",
                result=final_result,
                formula="分解减法：a - (b + c) = a - b - c"
            ))
            
        else:
            # 基本加法验算
            result = strategy['result']
            steps.append(CalculationStep(
                description=f"基本验算：{subtrahend} + {result} = {minuend}",
                operation="加法验算",
                result=result,
                formula="减法验算：a - b = c ⟺ b + c = a"
            ))
        
        return steps